#ifndef _DATACONTROLLER_H
#define _DATACONTROLLER_H

//Root includes
#include <TGFileDialog.h>
#include <TMultiGraph.h>
#include <TGStatusBar.h>
#include <TVirtualPad.h>
#include <TCanvas.h>
#include <TGraph.h>
#include <TH2.h>
#include <TH1.h>

//stdlib includes
#include <sstream>
#include <vector>
#include <string>
#define _USE_MATH_DEFINES
#include <math.h>

//NJE includes
#include "Global_Values_Extern.h"
#include "TomographyStructs.h"
#include "TomographyAnalysis.h"
//#include "TomographyStructDefaults.h"
#include "TomographyConfig.h"
#include "emittanceAnalysis.h"
#include "momentAnalysis.h"
#include "masterController.h"
#include "mainWindow.h"
#include "dataObject.h"
#include "MapClass.h"

class masterController;
class mainWindow;
class dataObject;

class dataController
{
	private:
		dataObject*        thisModel;
		emittanceAnalysis* thisEmittanceAnalysis;
		momentAnalysis*    thisMomentAnalysis;
		masterController*  master;

		void clearModel();
		
	public:

		dataController();
		~dataController();

		void setMaster( masterController* mc );
		bool isEmpty();
		bool isNotEmpty();	

		void tomography( MapClass* maps );
		void tomography( );
		void padData(const int halfPad);

		//Drawing functions
		void createHistogram(const int bunch, TVirtualPad* tvPad, const int type);
		void createRangeHistogram(const int bunch1, const int range, TVirtualPad* tvPad, const int type);
		
		void fillEmittanceGraph( const int bunch, const int range, TMultiGraph* graph );
		void fillEmittanceHisto( const int bunch, const int range, TH2F* hist );
		void fillHistogram(const int bunch, TH2F* hist, const int type);
		void fillRangeHistogram(const int bunch1, const int range, TH2F* hist, const int type);

		void fillImageMomentPhaseHistogram(const int order, TH1F* hist);
		void fillImageMomentPhaseFFTHistogram(const int order, TH1F* hist);

		void fillDiscrepancyGraph( const int bunch, const int range, TMultiGraph* graph );
		void fillDiscrepancyHisto( const int bunch, const int range, TH1F* hist );

		void drawContour_NanoSec(const double d_nanosec, TVirtualPad* tvPad, const double xOffset=0.0 );
		void drawContour_MeV(const double d_MeV, TVirtualPad* tvPad, const double xOffset=0.0 );
		void drawContour_Phi(const double d_Phi, TVirtualPad* tvPad, const double xOffset=0.0 );
		void drawRangeContours_Phi(const double d_Phi, TVirtualPad* tvPad, const int nBunches );
		void drawRangeContours_MeV(const double d_MeV, TVirtualPad* pad, const int nBunches);
		void drawRangeContours_NanoSec(const double d_nanosec, TVirtualPad* pad, const int nBunches);

		//Getters/setters
		complete_data_header_type getConfig();
		profile_data_header_type  getDataAcquisition();
		Tomography_Config         getTomography();
		Machine_Config            getMachine();
		std::string dataInfo();

		void setDataDataAquisition( const profile_data_header_type config );
		void setDataTomography( const Tomography_Config config );
		void setDataMachine( const Machine_Config config );

		bool readInPRPFile(const char* filename);
		void fillDataFrom2DVector( const std::vector< std::vector<float> > input, const profile_data_header_type head );
		bool fillFrom1DArray(float* start, profile_data_header_type* head);

		void writeReconstruction(const char* filename);
		
		enum arrayTypes
		{
			kImageArray,
			kProfileArray,
			kDataArray
		};

	ClassDef(dataController,0) 
};
#endif
