#include "masterController.h"

masterController::masterController(int howManyTabs): TQObject()
{
  	//default settings initialized
	numberOfTabs = howManyTabs;
	isInLoop     = false;
	copyMachine(    &defaultMainInjector, &currentMachine    );
	copyTomography( &defaultTomography  , &currentTomography );

	// These were not filling eKin and mass correctly leading to garbage and trouble.
	// make sure to set these up better.
	// Get initial settings from defaults
	// std::stringstream defaultPath;
	// defaultPath << TARDIS_Resources << "/defaultSettings.machine";
	// readMachineConfigFromFile( defaultPath.str().c_str(), &currentMachine );

	// defaultPath.seekp( 0, defaultPath.beg );
	// defaultPath << TARDIS_Resources << "/defaultSettings.tomography";
	// readTomographyConfigFromFile(defaultPath.str().c_str(), &currentTomography );

	//Controllers - pre-processing, scope, data, (post-processing soon)
	dataControl.resize( numberOfTabs );
	for( int i = 0; i < numberOfTabs; ++i)
	{
		dataControl[i] = new dataController();
		dataControl[i]->setMaster( this );
	}

	ppControl = new ppController( numberOfTabs );

	#ifndef _OFFLINE_MODE
		scopeControl = new scopeController( this, ppControl );

		checkACNET = new TTimer();
		checkACNET->Connect( "Timeout()", "masterController", this, "ACNETInterrupt()" );
	  	checkACNET->Start( 100, kFALSE );

	  	//Online-only connections
 	 	TQObject::Connect( "masterController", "toggleScope()", "scopeController", scopeControl, "externalScopeLock()" );
 	 	TQObject::Connect( "mainWindow", "Destroyed()"       , "masterController", this, "exitACNET()"                 );
	#endif

	//Views are handled as follows:
	//	Each view needs: a pad to draw to, a format specifier, a dataContoller, and a masterController
	//	-Default formats are populated and handed to the views in the constructors.
	//	-When a thumb or main window is created it emits it's pads. Master coordinates setting the
	//  appropriate view's pads with the values emitted by the window. Not the most elegant solution
	//  but it works pretty well.
	//  -rebuildView is a hard update and should only be called if the format has been modified.
	//  -If data is updated, but view stays the same 'update' should be called. A view will only update if
	//  it is 'old', i.e. out of date with the data in the controller.

	buildDefaultViewFormats();
	mainViewVector.clear(); //main Window View
	thumViewVector.clear(); //thumb Window View

	for(int i = 0; i < numberOfTabs; ++i)
	{
		mainViewVector.push_back( new mainView( NULL, &mainViewFormat, dataControl[i], this ) );
		thumViewVector.push_back( new mainView( NULL, &thumViewFormat, dataControl[i], this ) );
		TQObject::Connect( "masterController", "dataWasUpdated()", "mainView", mainViewVector[i], "update()"  );
		TQObject::Connect( "masterController", "dataWasUpdated()", "mainView", thumViewVector[i], "update()"  );
	}

	//Make connections
	//These four connections will call rebuild on the views. Otherwise only update should ever be called.
	TQObject::Connect( "mainWindow"      , "emitPad(int,TVirtualPad*)", "masterController", this, "setMainViewPad(int,TVirtualPad*)" );
	TQObject::Connect( "thumbWindow"     , "emitPad(int,TVirtualPad*)", "masterController", this, "setThumViewPad(int,TVirtualPad*)" );
	TQObject::Connect( "plotLayoutWindow", "viewModified()"           , "masterController", this, "rebuildMainView()"                );
	TQObject::Connect( "thumbWindow"     , "viewModified()"           , "masterController", this, "rebuildThumView()"                );

    //Master Connections from mainWindow
    TQObject::Connect( "mainWindow", "emitSignal(Int_t)" , "masterController", this, "mainWindowSignal(Int_t)" );

    //Master Connections from thumbWindow
    TQObject::Connect( "thumbWindow", "emitButton(Int_t)" , "masterController", this, "thumbWindowSignal(Int_t)" );

	//Need to make connections from masterController once we have a window.
	mainWinPtr = new mainWindow( gClient->GetRoot(), numberOfTabs ); //This emits all it's pads which viewVector will connect to.

	#ifdef _OFFLINE_MODE
		mainWinPtr->SetWindowName("Tomography And Related Diagnostics In Synchrotrons : Offline");
	#endif

	TQObject::Connect( "masterController", "scopeStatus(bool)"      , "mainWindow", mainWinPtr, "setScopeButton(bool)"    );
	TQObject::Connect( "masterController", "loopStatus(bool)"       , "mainWindow", mainWinPtr, "setLoopButton(bool)"     );
    TQObject::Connect( "masterController", "statusUpdate(char*,int)", "mainWindow", mainWinPtr, "setStatus(char*,int)"    );
    TQObject::Connect( "masterController", "makeTabCurrent(int)"    , "mainWindow", mainWinPtr, "setTab(int)"             );
    TQObject::Connect( "masterController", "updateInfo(int, char*)" , "mainWindow", mainWinPtr, "setInfo(int, char*)"     );

    TQObject::Connect( "masterController", "updateProgress(int)"    , "mainWindow", mainWinPtr, "setProgress(int)"        );
    TQObject::Connect( "masterController", "progressTaskName(char*)", "mainWindow", mainWinPtr, "setProgressStatus(char*)");

	//Connections from smaller windows
	TQObject::Connect( "fullConfig"           , "emitButton(int)", "masterController", this, "fullConfigSignal(int)");
	TQObject::Connect( "structInspectorWindow", "emitButton(int)", "masterController", this, "structInspectorWindowSignal(int)");

	thumbWinPtr = NULL;

	//call rebuild to prep views.
	rebuildMainView();
	rebuildThumView();

	setGlobalPalette( Current_Format.scheme_ID );


	updateProgress(50);
	progressTaskName("Initialized.");
}

masterController::~masterController()
{
	#ifndef _OFFLINE_MODE
		delete scopeControl;
	#endif

	delete ppControl;

	delete mainWinPtr;
	if( thumbWinPtr != NULL ) delete thumbWinPtr;
}

void masterController::setMainViewPad( int tab, TVirtualPad* pad )
{
	if( tab >= 0 && tab < mainViewVector.size() ) mainViewVector[tab]->setPad(pad);
}

void masterController::rebuildMainView()
{
	for( int i = 0; i < mainViewVector.size(); ++i ) mainViewVector[i]->buildView();
	updateAll();
}

void masterController::setThumViewPad( int tab, TVirtualPad* pad )
{
	if( tab >= 0 && tab < thumViewVector.size() ) thumViewVector[tab]->setPad(pad);
}

void masterController::rebuildThumView()
{
	for( int i = 0; i < thumViewVector.size(); ++i ){ thumViewVector[i]->buildView(); };
	updateAll();
}

void masterController::updateAll()
{
	char timeText[128];
	time_t rawtime;
	struct tm *timeinfo;
	time( &rawtime );
	timeinfo = localtime(&rawtime);
	sprintf(timeText, "Time: %s", asctime( timeinfo ) );

	for( int tab = 0; tab < mainViewVector.size(); ++tab ) updateTab(tab, &timeText[0]);
}

void masterController::updateTab(int tab, char* text)
{
	if( tab < dataControl.size() ) updateInfo(tab, text);
	dataWasUpdated();
}



void masterController::tomography(const int tab)
{
	// dataControl[tab].tomography();
	dataControl[tab]->tomography();
	mainViewVector[tab]->setViewIsOld();
	thumViewVector[tab]->setViewIsOld();
	updateAll();
}

void masterController::tomography(const int tab, MapClass* maps)
{
	// dataControl[tab].tomography( maps );
	dataControl[tab]->tomography( maps );
	mainViewVector[tab]->setViewIsOld();
	thumViewVector[tab]->setViewIsOld();
	updateAll();
}

void masterController::mainWindowSignal(Int_t type)
{
	int CurrentTab = mainWinPtr->getCurrentTab();

	switch(type)
	{
		case mainWindow::kButton_getScope:
		{
			isInLoop = isInLoop ? !isInLoop : isInLoop; //Kill loop before killing scope
			toggleScope();

			#ifndef _OFFLINE_MODE
				bool scopeReady = scopeControl->scopeIsReady();
				scopeStatus(scopeReady);
			#endif
		}
		break;

		case mainWindow::kButton_Tomo:
		{
			updateProgress(10);
			if( dataControl[CurrentTab]->isNotEmpty() )
			{
				tomography(CurrentTab);
			}else{
				statusUpdate("No data for this tab. Load file or start scope.");
			}
		}
		break;
		case mainWindow::kButton_ScopeSingle:
		{
			#ifndef _OFFLINE_MODE
				grabScopeData( CurrentTab );
				tomography(CurrentTab);
			#endif
		}
		break;
		case mainWindow::kButton_Mlti:
		{
			if( thumbWindow::getCount() == 0 )
			{
				thumbWinPtr = new thumbWindow( gClient->GetRoot(), 200, 200, this, &thumViewFormat );
				rebuildThumView();
			}else{
				thumbWinPtr->raise();
			}
		}
		break;
		case mainWindow::kButton_Hist:
		{
			new historyWindow( gClient->GetRoot(), 200, 200, this );
		}
		break;

		case mainWindow::kButton_Loop:
			#ifndef _OFFLINE_MODE
				loop(); //This the big one.
			#endif
		break;

		case mainWindow::kMenu_File_Open_PRP:
			openExistingDialog( mainWinPtr );
		break;
		case mainWindow::kMenu_File_Open_RAW:
			openRAWDialog( mainWinPtr, CurrentTab );
		break;
		case mainWindow::kMenu_File_Open_Ming_Jen:
			openMingJenDialog( mainWinPtr, CurrentTab );
		break;
		case mainWindow::kMenu_File_Save_Raw_Data:
			if( ppControl->isNotEmpty( CurrentTab ) )
			{
				if( ppControl->getDAQStruct( &currentDAQ, CurrentTab ) )
				{
					new structInspectorWindow( gClient->GetRoot(), &currentDAQ, 200, 200 );
					statusUpdate(&currentDAQ.title[0]);
					saveRAWDialog();
				}else{
					statusUpdate("No DAQ information. Hm.");
				}
			}else{
				statusUpdate("No raw data for this tab.");
			}
		break;
		case mainWindow::kMenu_File_Save_Recon:
			saveReconstructionsDialog();
		break;
		case mainWindow::kMenu_File_Save_Recon_Prof:
		break;
		case mainWindow::kMenu_File_Save_Canvas_PDF:
			saveCanvasPDFDialog( mainWinPtr->getCanvas(CurrentTab), mainWinPtr );
		break;
		case mainWindow::kMenu_Config_Scope:
		{
			#ifndef _OFFLINE_MODE
				scopeControl->launchWindow( gClient->GetRoot() );
			#endif
		}
		break;
		case mainWindow::kMenu_Config_Recon:
		{
			if( fullConfig::getCount() == 0 ) new fullConfig( gClient->GetRoot(), 300, 200, this );   //Size set in ctor.
		}
		break;
		case mainWindow::kMenu_Config_Format:
		{
			if( formatConfig::getCount() == 0 ) new formatConfig( gClient->GetRoot(), 300, 200, this );   //Size set in ctor.
		}
		break;
		case mainWindow::kMenu_Config_PlotLayout:
		{
			if( plotLayoutWindow::getCount() == 0 ) new plotLayoutWindow( gClient->GetRoot(), 300, 200, this, &mainViewFormat );   //Size set in ctor.
		}
		break;
		case mainWindow::kMenu_Input_Preprocessed:
			break;
		case mainWindow::kMenu_Input_MingJen:
			break;
		case mainWindow::kMenu_Algo_FCART: //These should be implemented in the config window now.
			currentTomography.algorithm = kFCART_ID;
			break;
		case mainWindow::kMenu_Algo_DFCART:
			currentTomography.algorithm = kDFCART_ID;
			break;
		case mainWindow::kMenu_Algo_FCART2:
			currentTomography.algorithm = kFCART2_ID;
			break;
	}
}

void masterController::thumbWindowSignal( int type )
{
	int CurrentTab = thumbWinPtr->getCurrentTab();
	switch(type)
	{
		case thumbWindow::kButton_Update:
			thumbWinPtr->createUserView();
			updateAll();
		break;

		case thumbWindow::kMenu_File_Save_Canvas_PDF:
			saveCanvasPDFDialog( thumbWinPtr->getCanvas(CurrentTab), thumbWinPtr );
		break;

		case thumbWindow::kMenu_Opts_Recon:
		case thumbWindow::kMenu_Opts_Recon_Data:
		case thumbWindow::kMenu_Opts_Recon_Prof_Data:
			thumbWinPtr->setPlotOption(type);
			break;
	}
}

void masterController::fullConfigSignal( int type )
{
	switch(type)
	{
		case fullConfig::kButton_mSet:
		case fullConfig::kButton_mSave:
		case fullConfig::kButton_mLoad:
		break;
		case fullConfig::kButton_mApply:
			applyCurrentMachineConfigToAllTabs();
		break;
		case fullConfig::kButton_mMakeDefault:
		{
			std::stringstream defaultPath;
			defaultPath << TARDIS_Resources << "/defaultSettings.machine";
			writeMachineConfigToFile( defaultPath.str().c_str(), currentMachine );
		}
		break;
		case fullConfig::kButton_tSet:
		case fullConfig::kButton_tSave:
		case fullConfig::kButton_tLoad:
		break;
		case fullConfig::kButton_tApply:
			applyCurrentReconConfigToAllTabs();
		break;
		case fullConfig::kButton_tMakeDefault:
		{
			std::stringstream defaultPath;
			defaultPath << TARDIS_Resources << "/defaultSettings.tomography";
			writeTomographyConfigToFile( defaultPath.str().c_str(), currentTomography );
		}
		break;
		default:
		break;
	}
}

void masterController::structInspectorWindowSignal(int type)
{
	switch(type)
	{
		case structInspectorWindow::kButton_Continue:
			statusUpdate(&currentDAQ.title[0]);
			saveRAWDialog();
		break;
		case structInspectorWindow::kButton_Cancel:
		default:
		break;
	}
}

void masterController::plotLayoutWindowSignal(int type)
{
	switch(type)
	{
		default:
		break;
	}
}

void masterController::updateHistory()
{
	;
}

void masterController::saveReconstructionsDialog()
{

	TGFileInfo fi;

	fi.fIniDir = StrDup( TARDIS_Saved.c_str() );
	char PlaceHolder[48];
	sprintf(PlaceHolder, "reconstructions.tom" );
	fi.fFilename = StrDup( (char *)PlaceHolder );

	new TGFileDialog( gClient->GetRoot(), mainWinPtr, kFDSave, &fi) ;
	int currentTab = mainWinPtr->getCurrentTab();

	// if( dataControl[currentTab].isNotEmpty() )
	if( dataControl[currentTab]->isNotEmpty() )
	{
		// if(fi.fFilename != NULL) dataControl[currentTab].writeReconstruction( fi.fFilename );
		if(fi.fFilename != NULL) dataControl[currentTab]->writeReconstruction( fi.fFilename );
	}else{
		char status[200];
		sprintf(status, "No data to write!");
		statusUpdate(status);
	}

}

void masterController::saveRAWDialog()
{

	TGFileInfo fi;

	fi.fIniDir = StrDup( TARDIS_Saved.c_str() );
	char PlaceHolder[48];
	sprintf(PlaceHolder, "rawData.bin" );
	fi.fFilename = StrDup( (char *)PlaceHolder );

	new TGFileDialog( gClient->GetRoot(), mainWinPtr, kFDSave, &fi) ;
	int currentTab = mainWinPtr->getCurrentTab();

	char status[512];
	//The ppController is a little different than the dataContoller
	//I might change this. master would have a vector of ppControllers.
	if( ppControl->isNotEmpty(currentTab) && fi.fFilename != NULL )
	{
		ppControl->writeReallyRawData( fi.fFilename, currentTab );
		sprintf(status, "Raw scope output written to:%s", fi.fFilename );
	}else if( fi.fFilename == NULL ){
		sprintf(status, "Save cancelled.");
	}else{
		sprintf(status, "No data to write!");
	}

	statusUpdate(status);

}

void masterController::saveCanvasPDFDialog(embeddedInfoCanvas* infoPad, const TGWindow* window)
{
	TCanvas* canvas = (infoPad->GetEmbedded())->GetCanvas();

	TGFileInfo fi;

	fi.fIniDir = StrDup( TARDIS_Saved.c_str() );
	char PlaceHolder[48];
	sprintf(PlaceHolder, "saveCanvas.pdf" );
	fi.fFilename = StrDup( (char *)PlaceHolder );

	new TGFileDialog( gClient->GetRoot(), window, kFDSave, &fi) ;

	if(fi.fFilename != NULL) canvas->SaveAs( fi.fFilename );
}

void masterController::saveCanvasPDFDialog(TCanvas* canvas, const TGWindow* window)
{
	TGFileInfo fi;

	fi.fIniDir = StrDup( TARDIS_Saved.c_str() );
	char PlaceHolder[48];
	sprintf(PlaceHolder, "saveCanvas.pdf" );
	fi.fFilename = StrDup( (char *)PlaceHolder );

	new TGFileDialog( gClient->GetRoot(), window, kFDSave, &fi );

	if(fi.fFilename != NULL) canvas->SaveAs( fi.fFilename );
}

void masterController::openExistingDialog( mainWindow* window )
{
	int CurrentTab = window->getCurrentTab();

	const char* filetypes[] = { "Pre-Processed", "*.prp", "All files", "*", 0 , 0 };
  	char status[256];

 	TGFileInfo fi;
 	fi.fFileTypes = filetypes;
	fi.fIniDir    = StrDup( TARDIS_Saved.c_str() );

	new TGFileDialog( gClient->GetRoot(), window, kFDOpen, &fi );

	if(fi.fFilename != NULL)
	{
		//Fill the appropriate data slot. If read fails, say so.
		// if( dataControl[ CurrentTab ].readInPRPFile(fi.fFilename) )
		if( dataControl[ CurrentTab ]->readInPRPFile(fi.fFilename) )
		{
			sprintf( status, "File:%s read in.", fi.fFilename );
		}else{
			sprintf( status, "%s is wrong format or corrupt.", fi.fFilename );
		}

		statusUpdate(status);
	}

}

void masterController::openRAWDialog( mainWindow* window, int currentTab )
{

	const char* filetypes[] = { "Raw", "*.bin", "All files", "*", 0 , 0 };
  	char status[256];

 	TGFileInfo fi;
 	fi.fFileTypes = filetypes;
	fi.fIniDir    = StrDup( TARDIS_Saved.c_str() );

	new TGFileDialog( gClient->GetRoot(), window, kFDOpen, &fi );

	ppStruct       preValues;
	profile_data_header_type localHeader;

	//Defaults from scopeController for now. Maybe very bad. A few are missing? Problem?
	preValues.bunchMidMethod     = quick  ;
	preValues.profileMethod      = simple ;
	preValues.bunchChargePartial = 0.05   ;
	preValues.bunchChargeFull    = 0.1    ;
	preValues.ptsPerProfile      = 95     ;

	preValues.sigTrig   = 1.5;
	preValues.dITrig    = 2.0;
	preValues.bunchTrig = 2.0;

	// This needs some protection if the file came from a preprocessed file and no
	// rawData exists.
	if(fi.fFilename != NULL)
	{
		if( ppControl->newDataFromRawFile( fi.fFilename, &currentMachine, &preValues, currentTab ) )
		{
			sprintf( status, "Raw file read in." );

			//Need to do preprocessing after this read in.
			std::vector<float> currentData(1);
			ppControl->analyseData( currentTab );

			ppControl->requestProfiles( currentData, &currentHeader, kRequestFull1, currentTab );

			dataControl[currentTab]->fillFrom1DArray( &currentData[0], &currentHeader );

			tomography( currentTab );
		}else{
			sprintf(status, "Oops. Error reading raw file.");
		}
		statusUpdate(status);
	}
}

void masterController::openMingJenDialog( mainWindow* window, int currentTab )
{

	const char* filetypes[] = { "All files", "*", 0 , 0 };
  	char status[256];

 	TGFileInfo fi;
 	fi.fFileTypes = filetypes;
	fi.fIniDir    = StrDup( TARDIS_Saved.c_str() );

	new TGFileDialog( gClient->GetRoot(), window, kFDOpen, &fi );

	ppStruct       preValues;
	profile_data_header_type localHeader;

	//Defaults from scopeController for now. Maybe very bad. A few are missing? Problem?
	preValues.bunchMidMethod     = quick  ;
	preValues.profileMethod      = simple ;
	preValues.bunchChargePartial = 0.1    ;
	preValues.bunchChargeFull    = 0.85   ;
	preValues.ptsPerProfile      = 47     ;

	preValues.sigTrig   = 1.5; //This might all need tweaking.
	preValues.dITrig    = 2.0; //This might all need tweaking.
	preValues.bunchTrig = 2.0; //This might all need tweaking.

	// This needs some protection if the file came from a preprocessed file and no
	// rawData exists.
	if(fi.fFilename != NULL)
	{
		if( ppControl->newDataFromMingJenFile( fi.fFilename, &currentMachine, &preValues, currentTab ) )
		{
			sprintf( status, "Ming-Jen file read in." );

			//Need to do preprocessing after this read in.
			std::vector< float > currentData(1);
			ppControl->analyseData( currentTab );
			ppControl->requestProfiles( currentData, &currentHeader, kRequestPartialStart, currentTab );

			dataControl[currentTab]->fillFrom1DArray( &currentData[0], &currentHeader );

			tomography(currentTab);
		}else{
			sprintf(status, "Oops. Error reading raw file.");
		}

		statusUpdate(status);
		DAQStruct dummyDAQ;
		ppControl->getDAQStruct( &dummyDAQ, currentTab);
		sprintf(status, "%d", dummyDAQ.DAQTrig.numTraces );
		statusUpdate(status);
	}
}

void masterController::closeThumbWindow()
{
	thumbWinPtr = NULL;

	for( int i = 0; i < thumViewVector.size(); ++i ) thumViewVector[i]->disconnectPad();

}

void masterController::buildDefaultViewFormats()
{

	mainViewFormat.resize(4);

	mainViewFormat[0].plotType = kData;
	mainViewFormat[0].padXMin       = 0.0;
	mainViewFormat[0].padYMin       = 0.0;
	mainViewFormat[0].padXMax       = 0.33;
	mainViewFormat[0].padYMax       = 0.5;
	mainViewFormat[0].startBunch    =   0;
	mainViewFormat[0].endBunch      =   5;
	mainViewFormat[0].bunchesPerRow =   5;
	mainViewFormat[0].momentOrder   =   0;
	mainViewFormat[0].plotSeparatrix = true;
	mainViewFormat[0].contours.clear();
	mainViewFormat[0].numberOfContours = mainViewFormat[0].contours.size();

	mainViewFormat[1].plotType = kProj;
	mainViewFormat[1].padXMin       = 0.33;
	mainViewFormat[1].padYMin       = 0.0;
	mainViewFormat[1].padXMax       = 0.66;
	mainViewFormat[1].padYMax       = 0.5;
	mainViewFormat[1].startBunch    =   0;
	mainViewFormat[1].endBunch      =   5;
	mainViewFormat[1].bunchesPerRow =   5;
	mainViewFormat[1].momentOrder   =   0;
	mainViewFormat[1].plotSeparatrix = true;
	mainViewFormat[1].contours.clear();
	mainViewFormat[1].numberOfContours = mainViewFormat[1].contours.size();

	mainViewFormat[2].plotType = kImage;
	mainViewFormat[2].padXMin       = 0.66;
	mainViewFormat[2].padYMin       = 0.0;
	mainViewFormat[2].padXMax       = 1.0;
	mainViewFormat[2].padYMax       = 0.5;
	mainViewFormat[2].startBunch    =   0;
	mainViewFormat[2].endBunch      =   5;
	mainViewFormat[2].bunchesPerRow =   5;
	mainViewFormat[2].momentOrder   =   0;
	mainViewFormat[2].plotSeparatrix = true;

	mainViewFormat[2].contours.clear();
	//mainViewFormat[2].contours.push_back(8.0);
	mainViewFormat[2].numberOfContours = mainViewFormat[2].contours.size();

	mainViewFormat[3].plotType = kEmittanceHisto;
	mainViewFormat[3].padXMin       = 0.0;
	mainViewFormat[3].padYMin       = 0.5;
	mainViewFormat[3].padXMax       = 1.0;
	mainViewFormat[3].padYMax       = 1.0;
	mainViewFormat[3].startBunch    =   0;
	mainViewFormat[3].endBunch      =   5;
	mainViewFormat[3].bunchesPerRow =   5;
	mainViewFormat[3].momentOrder   =   0;
	mainViewFormat[3].plotSeparatrix = true;

	mainViewFormat[3].contours.clear();
	mainViewFormat[3].numberOfContours = mainViewFormat[3].contours.size();


	thumViewFormat.resize(5);
	int bperrow = 17;
	int imax    = thumViewFormat.size();
	double step = 1.0/(double)imax;

	for(int i = 0; i < imax; ++i)
	{
		thumViewFormat[i].plotType      = kImage;
		thumViewFormat[i].padXMin       = 0.0;
		thumViewFormat[i].padYMin       = (imax-i)*step;
		thumViewFormat[i].padXMax       = 1.0;
		thumViewFormat[i].padYMax       = (imax-i-1)*step;
		thumViewFormat[i].startBunch    = i * bperrow;
		thumViewFormat[i].bunchesPerRow = bperrow;
		thumViewFormat[i].momentOrder   = 0;
		thumViewFormat[i].plotSeparatrix = false;

		thumViewFormat[i].contours.clear();
		//thumViewFormat[i].contours.push_back(8.0);
		thumViewFormat[i].numberOfContours = thumViewFormat[i].contours.size();
	}
}

void masterController::applyCurrentMachineConfigToAllTabs()
{
	for( int i = 0; i < numberOfTabs; ++i ) dataControl[i]->setDataMachine( currentMachine );
}

void masterController::applyCurrentReconConfigToAllTabs()
{
	for( int i = 0; i < numberOfTabs; ++i ) dataControl[i]->setDataTomography( currentTomography );
}

#ifndef _OFFLINE_MODE

	void masterController::loop()
	{
		isInLoop = !isInLoop; //Toggle value

		loopStatus( isInLoop );

		char   timeText[128];
		time_t rawtime;
		struct tm *timeinfo;
        bool grabbedData = false;
		
		std::string mapLocation = StrDup( TARDIS_Maps.c_str() );

		int CurrentTab = mainWinPtr->getCurrentTab();

		//While loop makes sure we have data.
		std::stringstream grabStatus;
		while( !grabbedData && isInLoop )
		{ 
			grabbedData = grabScopeData(CurrentTab);
			gSystem->ProcessEvents();

			grabStatus << "Last attempt to grab data " << (grabbedData ? "ok" : "failed");
			progressTaskName( grabStatus.str().c_str() );
			grabStatus.str("");
			grabStatus.clear();
		}

		//'while' may have been killed by exiting the loop, or getting data.
		if( grabbedData && isInLoop )
		{

			statusUpdate("Working on Maps...");
           	
			MapClass maps( dataControl[CurrentTab]->getConfig(), mapLocation, MapClass::kSparseSingleUse );

			std::stringstream task;
			task << "Maps ready. Waiting on event...";
			progressTaskName( task.str().c_str() );

			while( isInLoop )
			{
				//First time through the loop we have data.
				//We don't need to try and grab again until after
				//we've processed that data.
				if( grabbedData )
				{
					tomography( CurrentTab, &maps );

					time( &rawtime );
					timeinfo = localtime(&rawtime);
					sprintf(timeText, "Time: %s", asctime( timeinfo ) );

					updateTab(CurrentTab, &timeText[0]);

					makeTabCurrent( CurrentTab );
					CurrentTab = ( CurrentTab + 1 ) % numberOfTabs; //Tabs #'ed from 0
				}

				statusUpdate("In Loop: Waiting on next event...");
				grabbedData = grabScopeData(CurrentTab);

				gSystem->ProcessEvents();
			}
		}

		loopStatus( isInLoop );
	}

	bool masterController::grabScopeData(int tab)
	{
		bool proceed = false;
		std::stringstream whatHappened;

		if( scopeControl->scopeIsReady() )
		{

			float* data = scopeControl->getCurrentData( tab, &currentHeader, &proceed );

			if( proceed )
			{
				dataControl[tab]->fillFrom1DArray( data, &currentHeader );
				whatHappened << "scope data transferred.";
			}else{
				proceed = false;
				whatHappened << "Failed during data acquisition.";
			}
		}else{
			isInLoop = isInLoop ? !isInLoop : isInLoop; //If you're trying to loop stop!
			whatHappened << "The scope is not ready.";
		}

		statusUpdate( whatHappened.str().c_str() );

		return proceed;

	}
	void masterController::exitACNET()
	{
		checkACNET->Stop();
		term();
	  	exit(0);
	}

	void masterController::ACNETInterrupt()
	{
			static short int_wid;                                               /* interrupt window ID           */
		  	static short int_type;                                              /* interrupt type                */
		  	static int   int_row, int_col;                                      /* interrupt location data       */
		  	static int   int_info;                                              /* special interrupt information */
		  	window_intype(&int_wid, &int_type, &int_row, &int_col, &int_info);  /* retrieve next interrupt (event) */
	}
#endif


