#ifndef _MASTERCONTROLLER_H
#define _MASTERCONTROLLER_H
//#ifndef __CINT__
//#include <boost/filesystem.hpp>
//#include <boost/format.hpp>
//#endif
#include <vector>
#include <string>
#include <sstream>

#include "TSystem.h"

#include "TomographyDrawing.h"
#include "embeddedInfoCanvas.h"

#ifndef _OFFLINE_MODE
#include "scopeController.h"
#endif

#include "dataController.h"
#include "ppController.h"

#include "Global_Values_Extern.h"
#include "mainWindow.h"
#include "fullConfig.h"
#include "thumbWindow.h"
#include "formatConfig.h"
#include "historyWindow.h"
#include "plotLayoutWindow.h"
#include "structInspectorWindow.h"

#include "mainView.h"
#include "TQObject.h"
#include "TomographyStructs.h"
#include "TomographyConfig.h"
//#include "TomographyStructDefaults.h"

class TQObject;

#ifndef _OFFLINE_MODE
	class scopeController;
#endif

class mainView;
class mainWindow;
class thumbWindow;
class dataController;
class fullConfig; 
class formatConfig;
class historyWindow;
class plotLayoutWindow;
class ppController;
class structInspectorWindow;


class masterController: public TQObject
{
	private:
		int  numberOfTabs;
		bool isInLoop;

		TTimer      *checkACNET;

		//Global machine, tomography and profile headers. 
		profile_data_header_type currentHeader;     //profile header not used often since comes w/data from scope 
		Machine_Config           currentMachine;
		Tomography_Config        currentTomography;
		DAQStruct                currentDAQ; 

		std::vector< mainView* > mainViewVector; //vectors of mainViews
		std::vector< mainView* > thumViewVector; //vectors of mainViews

		std::vector< plotFormat > mainViewFormat; //formats for mainViews. Views and a windows will have pointers to these. 
		std::vector< plotFormat > thumViewFormat; //formats for mainViews. Views and a windows will have pointers to these. 

		//Controllers
		std::vector< dataController* > dataControl; //Several data sets per instance of program.
		ppController* ppControl;
		//std::vector< analysisController > analysisControl; 

		#ifndef _OFFLINE_MODE
			scopeController* scopeControl;             //Single scope per instance of program.
		#endif

		//Data view windows. Signal slot effectively acts like controller, master is enough.
		int mainCurrentTab;
		int thumCurrentTab;

		mainWindow*  mainWinPtr;
		thumbWindow* thumbWinPtr;

		void buildDefaultViewFormats();

	public:
		masterController( int howManyTabs );
		~masterController();

		//Handle All Window Signals
		void mainWindowSignal( Int_t type );
		void thumbWindowSignal( int type );
		void fullConfigSignal( int type );
		void plotLayoutWindowSignal( int type );
		void structInspectorWindowSignal( int type );
		
		void closeThumbWindow();	


		//Getters/Setters	
		int getNumberOfTabs(){ return numberOfTabs; };
		ppController* getPreProcessor(){ return ppControl; }; //I shouldn't need this

		Machine_Config    getCurrentMachine(){    return currentMachine;    };
		Tomography_Config getCurrentTomography(){ return currentTomography; };

		void setCurrentMachine( Machine_Config machine ){ copyMachine( &machine, &currentMachine ); };
		void setCurrentTomography( Tomography_Config tomography ){copyTomography( &tomography, &currentTomography ); };

		void setMainViewPad( int tab, TVirtualPad* pad );
		void setThumViewPad( int tab, TVirtualPad* pad );

		void setMainCurrentTab( int to ){ mainCurrentTab = to; };
		void setThumCurrentTab( int to ){ thumCurrentTab = to; };

		bool isLooping(){ return isInLoop; };
		
		//Looping
		void updateHistory();
		void tomography( const int tab );
		void tomography( const int tab, MapClass* maps );

		//Drawing
		void updateAll();
		void updateTab( int tab, char* text );
		void rebuildMainView();
		void rebuildThumView();
		
		//Dialogs
		void saveReconstructionsDialog();
		void saveRAWDialog();
		void saveCanvasPDFDialog( embeddedInfoCanvas* infoPad, const TGWindow* window ); //These should belong to tabbed info window
		void saveCanvasPDFDialog( TCanvas* canvas, const TGWindow* window );             //These should belong to tabbed info window 
		void openExistingDialog( mainWindow* window );
		void openMingJenDialog( mainWindow* window, int CurrentTab );
		void openRAWDialog( mainWindow* window, int CurrentTab );

		//Signals
		void statusUpdate( const char* status, int part = 0 ){ EmitVA( "statusUpdate(char*, int)", 2, status, part ); };//*Signal*
		void updateInfo( int tab, char* info){ EmitVA( "updateInfo(int, char*)", 2, tab, info ); };             //*Signal*
		void makeTabCurrent( int tab ){ EmitVA( "makeTabCurrent(int)", 1, tab ); };                             //*Signal*
		void scopeStatus( bool stat ){ EmitVA( "scopeStatus(bool)", 1, stat ); };	                            //*Signal*		
		void loopStatus( bool stat ){ EmitVA( "loopStatus(bool)", 1, stat ); };	                                //*Signal*
		void dataWasUpdated(){ Emit( "dataWasUpdated()" ); };                                                   //*Signal*
		void toggleScope(){ Emit( "toggleScope()" ); };     													//*Signal*

		void updateProgress(const int p){ EmitVA( "updateProgress(int)", 1, p ); };                             //*Signal*
		void progressTaskName(const char* n){ EmitVA( "progressTaskName(char*)", 1, n ); };                     //*Signal*

		void applyCurrentMachineConfigToAllTabs();
		void applyCurrentReconConfigToAllTabs();

		#ifndef _OFFLINE_MODE
			void loop();
			bool grabScopeData( int tab );
			void ACNETInterrupt();
			void exitACNET();
		#endif

	ClassDef(masterController, 0);
};

#endif

