#include "ppController.h"

ppController::ppController(const int howManyTabs)
{
    dataObjects.resize( howManyTabs, NULL );
    dataCount = 0;
}

ppController::~ppController()
{
    for(int i = 0 ; i < dataObjects.size(); ++i) //dtor
    {
        delete dataObjects[i]; ///which one??
        dataObjects[i] = NULL;
    }
}

bool ppController::newData(const unsigned char * inData, const Machine_Config * mcIN, const scopeParam * spIN, const ppStruct * ppsIN, int ID)
{
    /// create a new data object or overwrite an existing one
    ppDataObject *newdat = new ppDataObject( inData, mcIN, spIN, ppsIN ); // new object to add delete in destructor

    if( IDMap.find(ID) != IDMap.end() ) // if the ID is in the list replace it
    {
        delete dataObjects[ IDMap[ID] ];
        dataObjects[ IDMap[ID] ] = newdat;
        //Replacing old entry, dataCount unchanged

    }else if( dataObjects.size() < IDMap.size() ){

        IDMap[ID] = dataCount;
        dataObjects.push_back(newdat);
        ++dataCount; //If a new entry is added increase dataCount

    }else{

        IDMap[ID] = dataCount;
        dataObjects[IDMap[ID]] = newdat;
        ++dataCount; //If a new entry is added increase dataCount

    }

    return true;
}

bool ppController::newDataFromRawFile(const char* fileName, const Machine_Config* machine, const ppStruct* pps, int ID)
{
    bool success = false;
    //Really raw data is scopeParam, unsigned char data
    //Read header
    ifstream in(fileName, std::ifstream::binary );

    in.seekg( 0, in.end );
    int fileSize = in.tellg();
    in.seekg( 0, in.beg );

    if( fileSize >= sizeof( DAQStruct ) )
    {
        // scopeParam head;
        DAQStruct head;
        in.read( (char*)&head, sizeof(DAQStruct) );

        //Calculate and read data
        int length = head.DAQScope.numFrames * head.DAQScope.ptsPerFrame;

        if( fileSize >= length - sizeof(DAQStruct) ) //Not sure if this check is OK.
        {
            unsigned char* dataBuffer = new unsigned char[length];
            in.read( (char*)dataBuffer, length );

            newData( dataBuffer, machine, &head.DAQScope, pps ,ID );
            setTrigInfo( ID, &head.DAQTrig ); //This should be part of creating new data
            success = true;
        }
    }

    in.close();

    return success;
}

bool ppController::changeParam(const Machine_Config * mcIN, const scopeParam * spIN, const ppStruct * ppsIN, int ID)
{
    bool success = false;

    if(IDMap.find(ID) != IDMap.end() )
    {
        dataObjects[ IDMap[ID] ]->changeParam( mcIN, spIN, ppsIN);
        success = true;
    }

    return success;
}

void ppController::setTrigInfo( int ID, const trigParam* info )
{
    if(IDMap.find(ID) != IDMap.end() ) dataObjects[ IDMap[ID] ]->setTrigInfo( info );
}

int ppController::analyseData(int ID)
{
  bool ans = false; // number of datastructures found

  if(IDMap.find(ID) != IDMap.end())
  {
    ans = dataObjects[ IDMap[ID] ]->analyseData();
  }

  return ans;
}
void ppController::getAnalysisResults(std::vector<int>& foranswer, int ID)
{
    if(IDMap.find(ID) != IDMap.end())
    {
        dataObjects[ IDMap[ID] ]->getAnalysisResults(foranswer);
    }else{
        foranswer[0] = -1;
        foranswer.resize(1);
    }
}

bool ppController::requestProfiles(std::vector< float >& profiles, profile_data_header_type * profHeader, const profileRequests request, int ID)
{
    bool ans = false;
    if(IDMap.find(ID) != IDMap.end())
    {
        ans = dataObjects[ IDMap[ID] ]->profilesRequestHandler( profiles, profHeader, request );
        dataObjects[ IDMap[ID] ]->getHeaderTrigParams( profHeader );
    }

    return ans;
}

bool ppController::getPPDataObjectInfo(ppDataObjectInfo * ans, int ID)
{
    bool success = false;

    if(IDMap.find(ID) != IDMap.end())
    {
        *ans = dataObjects[ IDMap[ID] ]->getPPDataObjectInfo();
        success = true;
    }

    return success;
}

void ppController::writeReallyRawData(const char* fileName, int tab)
{
    if( isNotEmpty(tab) ) dataObjects[tab]->writeReallyRawData(fileName);
}

bool ppController::isEmpty(int tab)
{
    return (IDMap.find(tab) != IDMap.end() ) ? false: true;
}

bool ppController::isNotEmpty(int tab)
{
    return !isEmpty(tab);
}

bool ppController::getDAQStruct( DAQStruct* someStruct, int ID )
{
    bool success = false;
    if(IDMap.find(ID) != IDMap.end())
    {
     (*someStruct).DAQScope = (dataObjects[ IDMap[ID] ]->getPPDataObjectInfo()).sp;
     (*someStruct).DAQTrig  = (dataObjects[ IDMap[ID] ]->getPPDataObjectInfo()).ppTrigInfo;
     success = true;
    }

    return success;
}

bool ppController::newDataFromMingJenFile( const char* filename, const Machine_Config* machine, const ppStruct* pps, int ID )
{
    DAQStruct thisDAQ;
    header_type mingJenHeader;
    std::ifstream inFile( filename, std::ios::in | std::ios::binary );

    inFile.seekg (0, inFile.end);
    int realSize = inFile.tellg();
    inFile.seekg (0, inFile.beg);

    bool didRead = false;

    if( realSize >= sizeof( header_type ) ) // make sure it's at least as big as a header
    {
        //Don't read in a struct this way unless it's packed!
        inFile.read( (char*)&mingJenHeader, sizeof(header_type) );
        long NPointsTot = mingJenHeader.rtd_len * mingJenHeader.rtd_nrec; //Length of one bunch*DataFrames
        long dataSize   =  NPointsTot * sizeof(unsigned char);

        //MingJen adds time stamp for each frame at end. So you need to account for that
        //For now as long as real size is larger than data proceed
        if( realSize >= dataSize + sizeof(header_type) )
        {
            //Read all data into reallyRawData.
            inFile.seekg( sizeof( header_type ), inFile.beg );
            std::vector<unsigned char> reallyRawData( NPointsTot );
            inFile.read( (char*)&reallyRawData[0], NPointsTot ); //dataSize );

            mingJenHeaderToDAQStruct( &mingJenHeader, &thisDAQ );

            newData( &reallyRawData[0], machine, &thisDAQ.DAQScope, pps, ID );
            setTrigInfo( ID, &thisDAQ.DAQTrig );

            didRead = true;
        }
    }
    #ifdef _STANDALONE_VERBOSE
        if(didRead == false) std::cout << "File format incorrect or file is corrupt."  << std::endl;
    #endif

    inFile.close();

    return didRead;
}



void ppController::launchPPGUI( const TGWindow *p )
{
    if( !ppView )
    {
        //ppView = new ppGUI( p );
        ppView = new ppGUI( p , dataCount);
        actualGUIButtonState = GUIBUTTON;
    }
}

void ppController::ppGUISignalHandler(int signalID)
{
    toggleGUIButtons();
    switch(signalID)
    {
        case ppGUI::kButton_FindSig:
            #ifdef _PP_VERBOSE
              std::cout << "Attempting To Find Signal 2" << std::endl;
            #endif

            break;
        case ppGUI::kButton_FindInj:

            break;
        case ppGUI::kButton_Findbatch:

            break;
    };
    gSystem->ProcessEvents(); // Process all events with buttons disconnected
    toggleGUIButtons();

}


void ppController::toggleGUIButtons()
{
    if(actualGUIButtonState == GUIBUTTON)
    {
        if(ppView)
        {
            TQObject::Disconnect("ppGUI", "emitButton(int)", this, "ppGUIButton(int)"); // /Disconnect the scope gui buttons
        }
        actualGUIButtonState = GUIBUTTOFF;
    }
    else
    {
        if(ppView)
        {
            TQObject::Connect("ppGUI", "emitButton(int)", "ppController", this, "ppGUIButton(int)");
        }
       actualGUIButtonState = GUIBUTTON;
    }
}


