#ifndef _TomographyAnalysis_H_
#define _TomographyAnalysis_H_
#define _USE_MATH_DEFINES
#include <algorithm>
#include <numeric>
#include <math.h>
#include <vector>

#include <TVirtualFFT.h>
#include <TGraph.h>
#include <TH1.h>
#include <TF1.h>

#include "TomographyStructs.h"
//#include "TomographyStructDefaults.h"

/// This returns the Synchperiod, dvided by the order, useful for, say, quad oscilaltions going at twice the frequency etc...

// double GetSynchPeriod(int order, GlobalConfigType Global);

double GetSynchPeriod(int order, Machine_Config machine);

double linearinterp(double x1, double x2, double y1, double y2, double y3);

double GuessConst(std::vector<double>& yValues);

double GuessPeriod(std::vector<double>& xValues,std::vector<double>& yValues,int order, Machine_Config machine);

void GuessAmpandPhase(std::vector<double>& xValues, std::vector<double>& yValues,std::vector<double>& ansVec,double perGuess);

void FitDecayingCos(std::vector<double>& xValues,std::vector<double>& yValues,std::vector<double> & outVec,int order, Machine_Config machine);

void GetMom1and2(std::vector<int>& vecIn, double *first, double *second);

void GetMom1and2(std::vector<float>& vecIn, double *first, double *second);

void GetAveragedxData(std::vector<double>& vecOut, int turnspertrig,int startpos, int stoppos, int npToAverage);

void GetAveragedyData(std::vector<double>& vecIn, std::vector<double>& vecOut, int startpos, int stoppos, int npToAverage);

void GetProfDipQuadMomData( std::vector<int>& profData,profile_data_header_type& profHead, std::vector<double>& dipOut,std::vector<double>& quadOut, Machine_Config machine);

void GetProfDipQuadMomData( std::vector<float>& profData,profile_data_header_type& profHead, std::vector<double>& dipOut,std::vector<double>& quadOut, Machine_Config machine);

double Get1stMom(std::vector<double>& vecIn);

double SimpleTrackForDipAmp(double xMom,double yMom ,profile_data_header_type& profFileHeader, Machine_Config machine);

void GetReconDipMom(std::vector<float>& reconData,profile_data_header_type& profFileHeader,double *amp,  double *phase, Machine_Config machine);

void GetProfDipFFT(std::vector<double> &profDipMomData,  profile_data_header_type& profHead,std::vector<double> &vecOut);

void GetProfQuadFFT(std::vector<double> &profQuadMomData,  profile_data_header_type& profHead,std::vector<double> &vecOut);

void GetProfFFTMags(std::vector<double> &profDipMomData, std::vector<double> &profQuadMomData, profile_data_header_type& profHead, std::vector<double> &dipDataOut, std::vector<double> &quadDataOut);

void getArrayFFT(const std::vector<double> &input, const profile_data_header_type& profHead, std::vector<double> &output);

//Some of these have been overloaded to take GlobalConfigs and get rid of hardcoded values.
//The originals are below in case they need to be used. 
double GetSynchPeriod(int order);

double GuessPeriod(std::vector<double>& xValues,std::vector<double>& yValues,int order);

void FitDecayingCos(std::vector<double>& xValues,std::vector<double>& yValues,std::vector<double> & outVec,int order);

void GetProfDipQuadMomData( std::vector<int>& profData,profile_data_header_type& profHead, std::vector<double>& dipOut,std::vector<double>& quadOut);

double SimpleTrackForDipAmp(double xMom,double yMom ,profile_data_header_type& profFileHeader);

void GetReconDipMom(std::vector<float>& reconData,profile_data_header_type& profFileHeader,double *amp,  double *phase);

#endif
