#include "TomographyConfig.h"
#include <cmath>
#include <vector>
#include <cstdio>
#include <fstream>
#include <iostream>
//#include "TomographyStructDefaults.h"

//Structs read this way need to be packed. 
bool read_complete_data_header_fromFile( const char* fileName, complete_data_header_type* header )
{

  bool success = false;

  std::ifstream inputFile;
  inputFile.open(fileName, std::ios::in|std::ios::binary );

  inputFile.seekg( 0, inputFile.end );
  int length = inputFile.tellg();
  inputFile.seekg( 0, inputFile.beg );
  if(length == sizeof(complete_data_header_type))
  {
    inputFile.read( (char*)header, sizeof(complete_data_header_type));
    success = true;
  }
  
  inputFile.close();

  return success;
}

bool write_complete_data_header_toFile( const char* fileName, complete_data_header_type* header )
{
  bool success = false;

  std::ofstream outputFile;
  outputFile.open( fileName, std::ios::out|std::ios::binary );

  outputFile.write( (char*)header, sizeof(complete_data_header_type));
  
  outputFile.close();

  success = true;

  return success;
}

bool readMachineConfigFromFile( const char* fileName, Machine_Config* machine )
{

  bool success = 0;

  std::ifstream inputFile;
  inputFile.open(fileName, std::ios::in|std::ios::binary );

  inputFile.seekg( 0, inputFile.end );
  int length = inputFile.tellg();
  inputFile.seekg( 0, inputFile.beg );
  if(length >= sizeof(Machine_Config))
  {
    inputFile.read( (char*)machine, sizeof(Machine_Config));
    success = 1;
  }
  
  inputFile.close();

  return success;
}

bool writeMachineConfigToFile(const char* fileName, const Machine_Config machine)
{
  bool success = 0;

  std::ofstream outputFile;
  outputFile.open( fileName, std::ios::out|std::ios::binary );

  outputFile.write( (char*)&machine, sizeof(Machine_Config));
  
  outputFile.close();

  success = true;

  return success;
}

bool readTomographyConfigFromFile( const char* fileName, Tomography_Config* tomography )
{

  bool success = 0;

  std::ifstream inputFile;
  inputFile.open(fileName, std::ios::in|std::ios::binary );

  inputFile.seekg( 0, inputFile.end );
  int length = inputFile.tellg();
  inputFile.seekg( 0, inputFile.beg );
  if(length >= sizeof(Tomography_Config))
  {
    inputFile.read( (char*)tomography, sizeof(Tomography_Config));
    success = 1;
  }
  
  inputFile.close();

  return success;
}

bool writeTomographyConfigToFile(const char* fileName, const Tomography_Config tomography)
{
  bool success = 0;

  std::ofstream outputFile;
  outputFile.open( fileName, std::ios::out|std::ios::binary );

  outputFile.write( (char*)&tomography, sizeof(Tomography_Config));
  
  outputFile.close();

  success = true;

  return success;
}

void copyProfile_data_header_type( const profile_data_header_type* source, profile_data_header_type* destination)
{
  for(int i = 0; i < 60; ++i) destination->title[i] = source->title[i];
  for(int i = 0; i < 14; ++i) destination->time[i]  = source->time[i];
  destination->delay          = source->delay          ;    
  destination->turns_per_trig = source->turns_per_trig ;
  destination->time_last_meas = source->time_last_meas ;
  destination->hscale         = source->hscale         ;       
  destination->version        = source->version        ;      
  destination->num_bunches    = source->num_bunches    ;
  destination->num_frames     = source->num_frames     ;
  destination->ptsperprofile  = source->ptsperprofile  ;
}

void copyMachine( const Machine_Config* source, Machine_Config* destination)
{
  destination->h      = source->h      ; // [int]
  destination->eKin   = source->eKin   ; // [eV] 
  destination->mass   = source->mass   ; // [eV] change this to MeV
  destination->RF_V   = source->RF_V   ; // [V]
  destination->RF_f   = source->RF_f   ; // [Hz]
  destination->gammaT = source->gammaT ; // Real Number
  destination->synPhi = source->synPhi ; // ?rad? check maps
}

void copyTomography( const Tomography_Config* source, Tomography_Config* destination )
{
  destination->MaxIterations       = source->MaxIterations       ; // Max Iterations
  destination->SqrtPPP             = source->SqrtPPP             ; // Sqrt Particles per pixel
  destination->Frames              = source->Frames              ; // Number of Frames used
  destination->ReconstructionFrame = source->ReconstructionFrame ; // Where to Reconstruct limited to first frame for now. 
  destination->ConvergenceLimit    = source->ConvergenceLimit    ; // delta discrepancy turn-turn that signals convergence.
  destination->dampingFactor       = source->dampingFactor       ; // damping factor used in specific algorithm 
  destination->algorithm           = source->algorithm           ; // algorithm
  destination->pixelsPhi           = source->pixelsPhi           ; // pixelsPhi
  destination->pixelsdE            = source->pixelsdE            ; // pixelsdE
}


void print_profile_data_header_type( profile_data_header_type Head ){
  std::cout << "Data Acquisition:"                          << std::endl;
  std::cout << "\ttitle          = " << Head.title          << std::endl;
  std::cout << "\ttime           = " << Head.time           << std::endl;
  std::cout << "\tdelay          = " << Head.delay          << std::endl;
  std::cout << "\tturns_per_trig = " << Head.turns_per_trig << std::endl;
  std::cout << "\thscale         = " << Head.hscale         << std::endl;
  std::cout << "\tversion        = " << Head.version        << std::endl;
  std::cout << "\tnum_bunches    = " << Head.num_bunches    << std::endl;
  std::cout << "\tnum_frames     = " << Head.num_frames     << std::endl;
  std::cout << "\tptsperprofile  = " << Head.ptsperprofile  << std::endl;
};

void print_Tomography_Config( Tomography_Config tomography ){
  std::cout << "Tomography:"                                                << std::endl;
  std::cout << "\tMaxIterations       = " << tomography.MaxIterations       << std::endl;
  std::cout << "\tSqrtPPP             = " << tomography.SqrtPPP             << std::endl;
  std::cout << "\tFrames              = " << tomography.Frames              << std::endl;
  std::cout << "\tReconstructionFrame = " << tomography.ReconstructionFrame << std::endl;
  std::cout << "\tConvergenceLimit    = " << tomography.ConvergenceLimit    << std::endl;
  std::cout << "\tdampingFactor       = " << tomography.dampingFactor       << std::endl;
  std::cout << "\talgorithm           = " << tomography.algorithm           << std::endl;
  std::cout << "\tpixelsPhi           = " << tomography.pixelsPhi           << std::endl;
  std::cout << "\tpixelsdE            = " << tomography.pixelsdE            << std::endl;
}
void print_Machine_Config( Machine_Config machine ){
  std::cout << "Machine:"                       << std::endl;
  std::cout << "\th      = " << machine.h       << std::endl;
  std::cout << "\teKin   = " << machine.eKin    << std::endl;
  std::cout << "\tmass   = " << machine.mass    << std::endl;
  std::cout << "\tRF_V   = " << machine.RF_V    << std::endl;
  std::cout << "\tRF_f   = " << machine.RF_f    << std::endl;
  std::cout << "\tgammaT = " << machine.gammaT  << std::endl;
  std::cout << "\tsynPhi = " << machine.synPhi  << std::endl;
}

void print_complete_data_header_type(complete_data_header_type Config)
{

  print_profile_data_header_type( Config.dataAcquisition );
  
  print_Machine_Config( Config.machine );
  
  print_Tomography_Config( Config.tomography );
}

void clear_profile_data_header_type(profile_data_header_type head)
{

  head.title[0]       = 0;
  head.time[0]        = 0;
  head.delay          = 0;
  head.turns_per_trig = 0;
  head.hscale         = 0;
  head.version        = 0;
  head.num_bunches    = 0;
  head.num_frames     = 0;
  head.ptsperprofile  = 0;

}

void header_type_to_profile_data_header_type( header_type source, profile_data_header_type destination )
{
  
  for( int i = 0; i < 60; ++i ){ destination.title[i] = source.title[i]; }
  for( int i = 0; i < 14; ++i ){ destination.time[i]  = source.title[i]; }
  destination.delay           = source.delay;          // Trigger clock event delay (t0=us) 
  destination.turns_per_trig  = source.turns_per_trig; // Measured # of turns between trigs 
  destination.time_last_meas  = source.time_last_meas; // Time of last trigger - t0 (sec)   
  destination.hscale          = source.hscale;         // Horizontal scaling, ns/count      
  destination.version         = source.version;        // version # to track updates        
  destination.num_bunches     = 0;  // these three come from preprocessing.  
  destination.num_frames      = 0;  // these three come from preprocessing.
  destination.ptsperprofile   = 0;  // these three come from preprocessing.

}

void writeRawDataFile( const char* fileName, const DAQStruct* rawHeader, const unsigned char* data )
{
  std::ofstream out( fileName, std::ios::out|std::ios::binary );
    
  int totalNumberPoints = rawHeader->DAQScope.numFrames * rawHeader->DAQScope.ptsPerFrame;

  out.write( (char*)rawHeader, sizeof(DAQStruct)                         );//Write the header.
  out.write( (char*)data     , sizeof(unsigned char) * totalNumberPoints );//Write the data.
  out.close();
}

void mingJenHeaderToDAQStruct(const header_type* mingJenHeader, DAQStruct* TARDISHeader )
{
  sprintf(TARDISHeader->title, "%s", mingJenHeader->title);
  sprintf(TARDISHeader->time , "%s", mingJenHeader->time);
  TARDISHeader->version = -1; //-1 will mean mingJenType ? 

  TARDISHeader->DAQTrig.numTraces       = mingJenHeader->rtd_nrec;
  TARDISHeader->DAQTrig.turnsDelay      = mingJenHeader->turns_per_trig;
  //TARDISHeader->DAQTrig.beamSynchOffset = mingJenHeader->;// ???????
  TARDISHeader->DAQTrig.trigDelay       = mingJenHeader->delay;  
  TARDISHeader->DAQTrig.trigEvents[0]   = mingJenHeader->reset;
  //TARDISHeader->DAQTrig.trigState       = mingJenHeader->;
  //TARDISHeader->DAQTrig.padding[512]    = mingJenHeader->;

  TARDISHeader->DAQScope.numFrames    = mingJenHeader->rtd_nrec; //number of records
  //TARDISHeader->DAQScope.range        = mingJenHeader->;       //vscale
  //TARDISHeader->DAQScope.level        = mingJenHeader->;       //trigger level
  //TARDISHeader->DAQScope.offset       = mingJenHeader->;       //DC offset
  //TARDISHeader->DAQScope.channel      = mingJenHeader->;       //ok
  TARDISHeader->DAQScope.interval     = mingJenHeader->hscale;   //hscale
  TARDISHeader->DAQScope.ptsPerFrame  = mingJenHeader->rtd_len; //raw pts/frame from scope, record length
  //TARDISHeader->DAQScope.padding[512] = ;

}
