#include "TomographyDrawing.h"

#include <iostream>
#include <sstream>
#include <TLine.h>
#include <string>
#include <TH2.h>
#include <TF1.h>
#include <TColor.h>
#include <TGraph.h>
#include <TGaxis.h>
#include <TStyle.h>
#include <TPaveStats.h>
#include <vector>
#define _USE_MATH_DEFINES
#include <math.h>

#include "TomographyStructs.h"
//#include "TomographyStructDefaults.h"

void SetDipMomGraphStyle(TH1D *h1,TH1D *h2,TGraph *g1,TGraph *g2,TGaxis *axis, Format_Config Format){
    /// some parameters

    Double_t barWidth = 0.5;
    int h1Colour = Format.dipole_color;//2;
    int h2Colour = Format.quadrupole_color;//4;

    h1->SetMinimum(-3.15);

    /// Plot Style OPTIONS, Titles, Labels etc
    gStyle->SetOptStat("");  /// Remove stats box
    h1->SetTitle("Dipole Mode Amplitude and Phase");
    h1->GetXaxis()->SetTitle("Bunch Number");
    h1->GetYaxis()->SetTitle("Dip. Mom. Amp (bins)");
    h1->GetXaxis()->CenterTitle(1);
    h1->GetYaxis()->CenterTitle(1);
    h1->GetYaxis()->SetLabelColor(h2Colour);
    h1->GetYaxis()->SetTitleColor(h2Colour);
    h1->SetTitleSize(0.05,"XY");
    h1->SetTitleOffset(0.5,"Y");
    h1->SetLabelSize(0.05,"XY");
    h1->SetAxisColor(4, "Y");
    h1->SetTickLength(0.005,"XY");
    h1->SetFillColor(h1Colour);

    h2->SetFillColor(h2Colour);
    h1->SetBarWidth(0.4);
    h2->SetBarWidth(0.4);

    h1->SetBarOffset(0.10);
    h2->SetBarOffset(0.50);

    g1->SetMarkerColor(16);
    g1->SetMarkerStyle(34);
    g1->SetMarkerSize(0.875);
    g2->SetMarkerColor(16);
    g2->SetMarkerStyle(20);
    g2->SetMarkerSize(0.875);

    axis->SetLineColor(h1Colour);
    axis->SetLabelColor(h1Colour);
    axis->SetTextColor(h1Colour);
    axis->SetTickSize(0.002);
    axis->SetLabelSize(0.05);
    axis->SetTitle("Dip. Mom. Phase (#pi rad)");  /// # is for latex
    axis->SetTitleSize(0.05);
    axis->SetTitleOffset(1.2);
}

void SetDipMomGraphStyle(TH1D *h1,TH1D *h2,TGraph *g1,TGraph *g2,TGaxis *axis){
    /// some parameters

    Double_t barWidth =0.5;
    int h1Colour = 2;
    int h2Colour = 4;

    h1->SetMinimum(-3.15);

    /// Plot Style OPTIONS, Titles, Labels etc
    gStyle->SetOptStat("");  /// Remove stats box
    h1->SetTitle("Dipole Mode Amplitude and Phase");
    h1->GetXaxis()->SetTitle("Bunch Number");
    h1->GetYaxis()->SetTitle("Dip. Mom. Amp (bins)");
    h1->GetXaxis()->CenterTitle(1);
    h1->GetYaxis()->CenterTitle(1);
    h1->GetYaxis()->SetLabelColor(h2Colour);
    h1->GetYaxis()->SetTitleColor(h2Colour);
    h1->SetTitleSize(0.05,"XY");
    h1->SetTitleOffset(0.5,"Y");
    h1->SetLabelSize(0.05,"XY");
    h1->SetAxisColor(4, "Y");
    h1->SetTickLength(0.005,"XY");
    h1->SetFillColor(h1Colour);

    h2->SetFillColor(h2Colour);
    h1->SetBarWidth(0.4);
    h2->SetBarWidth(0.4);

    h1->SetBarOffset(0.10);
    h2->SetBarOffset(0.50);

    g1->SetMarkerColor(16);
    g1->SetMarkerStyle(34);
    g1->SetMarkerSize(0.875);
    g2->SetMarkerColor(16);
    g2->SetMarkerStyle(20);
    g2->SetMarkerSize(0.875);

    axis->SetLineColor(h1Colour);
    axis->SetLabelColor(h1Colour);
    axis->SetTextColor(h1Colour);
    axis->SetTickSize(0.002);
    axis->SetLabelSize(0.05);
    axis->SetTitle("Dip. Mom. Phase (#pi rad)");  /// # is for latex
    axis->SetTitleSize(0.05);
    axis->SetTitleOffset(1.2);
}

void SetQuadMomGraphStyle(TH1D *h1,TH1D *h2,TGraph *g1,TGraph *g2,TGaxis *axis, Format_Config Format)
{
    /// some parameters

    Double_t barWidth = 0.5;
    int h1Colour = Format.dipole_color;//2;
    int h2Colour = Format.quadrupole_color;//4;

    h1->SetMinimum(-3.15);

    /// Plot Style OPTIONS, Titles, Labels etc
    gStyle->SetOptStat("");  /// Remove stats box
    h1->SetTitle("Quadrupole Mode Amplitude and Phase");
    h1->GetXaxis()->SetTitle("Bunch Number");
    h1->GetYaxis()->SetTitle("Quad. Mom. Amp (bins)");
    h1->GetXaxis()->CenterTitle(1);
    h1->GetYaxis()->CenterTitle(1);
    h1->GetYaxis()->SetLabelColor(h2Colour);
    h1->GetYaxis()->SetTitleColor(h2Colour);
    h1->SetTitleSize(0.05,"XY");
    h1->SetTitleOffset(0.5,"Y");
    h1->SetLabelSize(0.05,"XY");
    h1->SetAxisColor(4, "Y");
    h1->SetTickLength(0.005,"XY");
    h1->SetFillColor(h1Colour);

    h2->SetFillColor(h2Colour);
    h1->SetBarWidth(0.4);
    h2->SetBarWidth(0.4);

    h1->SetBarOffset(0.10);
    h2->SetBarOffset(0.50);

    g1->SetMarkerColor(16);
    g1->SetMarkerStyle(34);
    g1->SetMarkerSize(0.875);
    g2->SetMarkerColor(16);
    g2->SetMarkerStyle(20);
    g2->SetMarkerSize(0.875);

    axis->SetLineColor(h1Colour);
    axis->SetLabelColor(h1Colour);
    axis->SetTextColor(h1Colour);
    axis->SetTickSize(0.002);
    axis->SetLabelSize(0.05);
    axis->SetTitle("Quad. Mom. Phase (#pi rad)");  /// # is for latex
    axis->SetTitleSize(0.05);
    axis->SetTitleOffset(0.5);
    axis->SetTitleOffset(1.2);
}

void SetQuadMomGraphStyle(TH1D *h1,TH1D *h2,TGraph *g1,TGraph *g2,TGaxis *axis){
    /// some parameters

    Double_t barWidth =0.5;
    int h1Colour = 2;
    int h2Colour = 4;

    h1->SetMinimum(-3.15);

    /// Plot Style OPTIONS, Titles, Labels etc
    gStyle->SetOptStat("");  /// Remove stats box
    h1->SetTitle("Quadrupole Mode Amplitude and Phase");
    h1->GetXaxis()->SetTitle("Bunch Number");
    h1->GetYaxis()->SetTitle("Quad. Mom. Amp (bins)");
    h1->GetXaxis()->CenterTitle(1);
    h1->GetYaxis()->CenterTitle(1);
    h1->GetYaxis()->SetLabelColor(h2Colour);
    h1->GetYaxis()->SetTitleColor(h2Colour);
    h1->SetTitleSize(0.05,"XY");
    h1->SetTitleOffset(0.5,"Y");
    h1->SetLabelSize(0.05,"XY");
    h1->SetAxisColor(4, "Y");
    h1->SetTickLength(0.005,"XY");
    h1->SetFillColor(h1Colour);

    h2->SetFillColor(h2Colour);
    h1->SetBarWidth(0.4);
    h2->SetBarWidth(0.4);

    h1->SetBarOffset(0.10);
    h2->SetBarOffset(0.50);

    g1->SetMarkerColor(16);
    g1->SetMarkerStyle(34);
    g1->SetMarkerSize(0.875);
    g2->SetMarkerColor(16);
    g2->SetMarkerStyle(20);
    g2->SetMarkerSize(0.875);

    axis->SetLineColor(h1Colour);
    axis->SetLabelColor(h1Colour);
    axis->SetTextColor(h1Colour);
    axis->SetTickSize(0.002);
    axis->SetLabelSize(0.05);
    axis->SetTitle("Quad. Mom. Phase (#pi rad)");  /// # is for latex
    axis->SetTitleSize(0.05);
    axis->SetTitleOffset(0.5);
    axis->SetTitleOffset(1.2);
}

void SetProfFFTGraphStyle(TH1D *h1,TH1D *h2, Format_Config Format)
{
    /// some parameters

    Double_t barWidth = 0.5;
    int h1Colour = Format.dipole_color;    //2;
    int h2Colour = Format.quadrupole_color;//4;

    //h1->SetMinimum(-3.15);
    h1->SetMinimum(0);
    /// Plot Style OPTIONS, Titles, Labels etc
    gStyle->SetOptStat("");  /// Remove stats box
    h1->SetTitle("Dipole, Quad Mode Amplitude");
    h1->GetXaxis()->SetTitle("Mode");
    h1->GetYaxis()->SetTitle("Amp (AU)");
    h1->GetXaxis()->CenterTitle(1);
    h1->GetYaxis()->CenterTitle(1);
    h1->GetYaxis()->SetLabelColor(h1Colour);
    h1->GetYaxis()->SetTitleColor(h1Colour);
    h1->SetTitleSize(0.05,"XY");
    h1->SetTitleOffset(1.0,"Y");
    h1->SetLabelSize(0.05,"XY");
    h1->SetAxisColor(4, "Y");
    h1->SetTickLength(0.005,"XY");
    h1->SetFillColor(h1Colour);

    h2->SetFillColor(h2Colour);
    h1->SetBarWidth(0.4);
    h2->SetBarWidth(0.4);

    h1->SetBarOffset(0.10);
    h2->SetBarOffset(0.50);
}

void SetProfFFTGraphStyle(TH1D *h1,TH1D *h2)
{
    /// some parameters

    Double_t barWidth =0.5;
    int h1Colour = 2;
    int h2Colour = 4;

    h1->SetMinimum(-3.15);

    /// Plot Style OPTIONS, Titles, Labels etc
    gStyle->SetOptStat("");  /// Remove stats box
    h1->SetTitle("Dipole, Quad Mode Amplitude");
    h1->GetXaxis()->SetTitle("Mode");
    h1->GetYaxis()->SetTitle("Amp (AU)");
    h1->GetXaxis()->CenterTitle(1);
    h1->GetYaxis()->CenterTitle(1);
    h1->GetYaxis()->SetLabelColor(h2Colour);
    h1->GetYaxis()->SetTitleColor(h2Colour);
    h1->SetTitleSize(0.05,"XY");
    h1->SetTitleOffset(0.5,"Y");
    h1->SetLabelSize(0.05,"XY");
    h1->SetAxisColor(4, "Y");
    h1->SetTickLength(0.005,"XY");
    h1->SetFillColor(h1Colour);

    h2->SetFillColor(h2Colour);
    h1->SetBarWidth(0.4);
    h2->SetBarWidth(0.4);

    h1->SetBarOffset(0.10);
    h2->SetBarOffset(0.50);
}


void DipMomGraph( std::vector<double>& profDipMomData, std::vector<double>& reconDipMomData, profile_data_header_type& profHead, TH1D *h1,TH1D *h2,TGraph *g1,TGraph *g2,TGaxis *axis)
{
    //const double pi =3.1415;
    /// Break-up the input data into seperate Vectors
    std::vector<double> profDipAmps(profHead.num_bunches);     /// fill with fitted amps
    std::vector<double> profDipAmpsErr(profHead.num_bunches);  /// fill with fitted amps errors
    std::vector<double> profDipPhases(profHead.num_bunches);   /// Fill with fitted phases
    std::vector<double> profDipPhasesErr(profHead.num_bunches);/// Fill with fitted phase errros
    std::vector<double> profDipChi2(profHead.num_bunches);     /// Fill with fit Chi^2

    std::vector<double> reconDipAmps(profHead.num_bunches);     /// fill with fitted amps
    std::vector<double> reconDipPhases(profHead.num_bunches);     /// fill with fitted amps

    for(int i=0;i<profDipAmps.size();++i)
    {
        profDipAmps[i]      = profDipMomData[i*5];
        profDipAmpsErr[i]   = profDipMomData[i*5+1];
        profDipPhases[i]    = profDipMomData[i*5+2];
        profDipPhasesErr[i] = profDipMomData[i*5+3];
        profDipChi2[i]      = profDipMomData[i*5+4];

        reconDipAmps[i]=reconDipMomData[i*2];
        reconDipPhases[i]=reconDipMomData[i*2+1];
    }

    /// Y axis range
    int pDipAmpsMax = (int)ceil(*max_element(profDipAmps.begin(), profDipAmps.end()));
    int pDipPhaseMax = (int)ceil(*max_element(profDipPhases.begin(), profDipPhases.end()));
    h1->SetMaximum((pDipAmpsMax>pDipPhaseMax ? pDipAmpsMax : pDipPhaseMax));

    ///Fill histograms

    for( int i=1;i<profDipAmps.size()+1;++i)
    {
        h1->Fill((Double_t)i-1,(Double_t)profDipPhases[i-1]);
        h2->Fill((Double_t)i-1,(Double_t)profDipAmps[i-1]);
        g1->SetPoint(i,i-0.7,reconDipPhases[i-1]);
        g2->SetPoint(i,i-0.3,reconDipAmps[i-1]);
        //std::cout << "reconDipAmps  " << reconDipAmps[i-1] << std::endl;
    }
    std::cout<<"pp2 "<<std::endl;
}

void QuadMomGraph(std::vector<double>& profQuadMomData, std::vector<double>& reconQuadMomData, profile_data_header_type& profHead, TH1D *h1,TH1D *h2,TGraph *g1,TGraph *g2,TGaxis *axis)
{
    //const double pi = 3.1415;
    /// Break-up the input data into seperate Vectors
    std::vector<double> profQuadAmps(profHead.num_bunches);     /// fill with fitted amps
    std::vector<double> profQuadAmpsErr(profHead.num_bunches);  /// fill with fitted amps errors
    std::vector<double> profQuadPhases(profHead.num_bunches);   /// Fill with fitted phases
    std::vector<double> profQuadPhasesErr(profHead.num_bunches);/// Fill with fitted phase errros
    std::vector<double> profQuadChi2(profHead.num_bunches);     /// Fill with fit Chi^2

    std::vector<double> reconQuadAmps(profHead.num_bunches);     /// fill with fitted amps
    std::vector<double> reconQuadPhases(profHead.num_bunches);     /// fill with fitted amps

    for(int i=0;i<profQuadAmps.size();++i)
    {
        profQuadAmps[i]      = profQuadMomData[i*5];
        profQuadAmpsErr[i]   = profQuadMomData[i*5+1];
        profQuadPhases[i]    = profQuadMomData[i*5+2];
        profQuadPhasesErr[i] = profQuadMomData[i*5+3];
        profQuadChi2[i]      = profQuadMomData[i*5+4];

        reconQuadAmps[i]=reconQuadMomData[i*2];
        reconQuadPhases[i]=reconQuadMomData[i*2+1];
    }

    /// Y axis range
    int pQuadAmpsMax = (int)ceil(*max_element(profQuadAmps.begin(), profQuadAmps.end()));
    int pQuadPhaseMax = (int)ceil(*max_element(profQuadPhases.begin(), profQuadPhases.end()));
    h1->SetMaximum((pQuadAmpsMax > pQuadPhaseMax ? pQuadAmpsMax : pQuadPhaseMax));

    ///Fill histograms

    for( int i=1;i<profQuadAmps.size()+1;++i)
    {
        h1->Fill((Double_t)i-1,(Double_t)profQuadPhases[i-1]);
        h2->Fill((Double_t)i-1,(Double_t)profQuadAmps[i-1]);
        g1->SetPoint(i,i-0.7,reconQuadPhases[i-1]);
        g2->SetPoint(i,i-0.3,reconQuadAmps[i-1]);
        //std::cout << "reconQuadAmps  " << reconQuadAmps[i-1] << std::endl;
    }
    std::cout << "pp2 " <<std::endl;
}


void ProfPhaseFFTGraph(
               std::vector<double>& profDipPhaseFFTMag,
               std::vector<double>& profQuadPhaseFFTMag,
               profile_data_header_type& profHead,
               TH1D *h1,TH1D *h2){
    //const double pi =3.1415;

    /// Y axis range
    int profDipPhaseFFTMagMax  = (int)ceil(*max_element(profDipPhaseFFTMag.begin(), profDipPhaseFFTMag.end()));
    int profQuadPhaseFFTMagMax = (int)ceil(*max_element(profQuadPhaseFFTMag.begin(), profQuadPhaseFFTMag.end()));
    h1->SetMaximum((profDipPhaseFFTMagMax>profQuadPhaseFFTMagMax ? profDipPhaseFFTMagMax : profQuadPhaseFFTMagMax));

    ///Fill histograms

    for( int i=1;i<profDipPhaseFFTMag.size()+1;++i)
    {
        h1->Fill((Double_t)i-1,(Double_t)profDipPhaseFFTMag[i-1]);
        h2->Fill((Double_t)i-1,(Double_t)profQuadPhaseFFTMag[i-1]);
    }
    std::cout<<"pp2 "<<std::endl;
}

void FormatRecon(TH2F* Recon){

  TPaveStats *Stats = (TPaveStats*)Recon->FindObject("stats");
  Stats->SetX1NDC(0.70);
  Stats->SetX2NDC(0.85);
  Stats->SetY1NDC(0.8);
  Stats->SetY2NDC(0.94);
  Stats->SetLineColor(0);
  Recon->GetXaxis()->SetLabelColor(13);
  Recon->GetYaxis()->SetLabelColor(13);
}

void DrawReticule(float centerX, float centerY, float RangeX, float RangeY, int RootColor){
  //Draws a reticule centered at (centerX,centerY) over RangeX, RangeY. N.B. Range not in bins. Actual
  //histo plotting range. dE, ns, etc. 
  TLine *R1 = new TLine( centerX, centerY-RangeY/2.0, centerX, centerY+RangeY/2.0 );
  TLine *R2 = new TLine( centerX-RangeX/2.0, centerY, centerX+RangeX/2.0, centerY );
  R1->SetLineColor((Int_t)RootColor);
  R2->SetLineColor((Int_t)RootColor);
  R1->SetLineStyle(1);
  R2->SetLineStyle(1);
  R1->Draw();
  R2->Draw();
}

void DrawPhaseSpaceCurve(Double_t BHeight, Double_t BSpc){
    //Make this more general so you can specify the max phi of the curve or something. 
    //TF1 *f3 = new TF1( "f3", "-1*[0]/1.41421*sqrt(-1/2-cos(2*3.141592*(x-[1]/2.0)/[1])) ", -0.36*BSpc/2.0, 0.36*BSpc/2.0 );
    //TF1 *f4 = new TF1( "f4", "   [0]/1.41421*sqrt(-1/2-cos(2*3.141592*(x-[1]/2.0)/[1])) ", -0.36*BSpc/2.0, 0.36*BSpc/2.0 );
    TF1 *f1 = new TF1( "f1", "-1*[0]*sin(3.141592*(x-[1]/2.0)/[1])", -BSpc/2.0, BSpc/2.0);
    TF1 *f2 = new TF1( "f2", "   [0]*sin(3.141592*(x-[1]/2.0)/[1])", -BSpc/2.0, BSpc/2.0);
    
    f1->SetLineColor(46); f1->SetLineWidth(2);
    f1->SetParameters( BHeight, BSpc); 
    f1->Draw("same");

    f2->SetLineColor(46); f2->SetLineWidth(2);
    f2->SetParameters( BHeight, BSpc);
    f2->Draw("same");
}

void DrawPhaseSpaceCurve(Double_t BHeight, Double_t BSpc, Double_t d_nanosec, int RootColor){
    
    double HalfRange = d_nanosec * ( 1 + 0.01 ); //BSpc*dPhiMax/6.28318530718*(1+0.05);
    double dPhiMax =  d_nanosec/BSpc * 2 * M_PI; //6.28318530718;

    TF1 *f1 = new TF1( "f1", "-1*[0]/1.41421*sqrt(-cos([2])-cos((x-[1]/2.0)*6.28318530718/[1])) ", -HalfRange, HalfRange );
    TF1 *f2 = new TF1( "f2", "   [0]/1.41421*sqrt(-cos([2])-cos((x-[1]/2.0)*6.28318530718/[1])) ", -HalfRange, HalfRange );
     
    f1->SetLineColor(RootColor); f1->SetLineWidth(2);
    f1->SetParameters( BHeight, BSpc, dPhiMax); 
    f1->Draw("same");

    f2->SetLineColor(RootColor); f2->SetLineWidth(2);
    f2->SetParameters( BHeight, BSpc, dPhiMax);
    f2->Draw("same");
}

void setGlobalPalette( int paletteNum ){

    const Int_t NumberOfContours = 255;
    
    switch(paletteNum)
    {
        case 0:
        {
            const Int_t NRGBs = 11;
            const Int_t NCont = 255;

            Double_t stops[NRGBs] = { 0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1. };
            Double_t red[NRGBs]   = { 0.77206100, 0.57616660, 0.38027220, 0.27294620, 0.25418860, 0.23543100, 0.26358420, 0.29173740, 0.27823320, 0.22307160, 0.16791000};
            Double_t green[NRGBs] = { 0.92462000, 0.81951920, 0.71441840, 0.59502440, 0.46133720, 0.32765000, 0.22089488, 0.11413976, 0.04860976, 0.02430488, 0.00000000};
            Double_t blue[NRGBs]  = { 0.99870300, 0.98845460, 0.97820620, 0.94512380, 0.88920740, 0.83329100, 0.74075380, 0.64821660, 0.54189260, 0.42178180, 0.30167100};
            TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, NCont);
        };
        break;
        case 1:
        {
            const Int_t NRGBs = 11;
            const Int_t NCont = 255;

            Double_t stops[NRGBs] = { 0., 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1. };
            Double_t red[NRGBs]   = { 0.46874200,  0.61009680,  0.75145160,  0.85169580,  0.91082940,  0.96996300,  0.98197780,  0.99399260,  1.00000000,  1.00000000,  1.00000000};
            Double_t green[NRGBs] = { 0.00000000,  0.04889000,  0.09778000,  0.17299620,  0.27453860,  0.37608100,  0.48442020,  0.59275940,  0.68156860,  0.75084780,  0.82012700};
            Double_t blue[NRGBs]  = { 0.01582360,  0.01107652,  0.00632944,  0.00962234,  0.02095522,  0.03228810,  0.05144122,  0.07059434,  0.08952772,  0.10824136,  0.12695500};

            TColor::CreateGradientColorTable(NRGBs,stops,red,green,blue,NCont);
        };
        break;
        case 2:
        {
            const Int_t NRGBs = 11;
            const Int_t NCont = 255;

            Double_t stops[NRGBs] = { 0.,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1. };
            Double_t red[NRGBs]   = { 0.92462000, 0.81951920, 0.71441840, 0.59502440, 0.46133720, 0.32765000, 0.22089488, 0.11413976, 0.04860976, 0.02430488, 0.00000000};
            Double_t green[NRGBs] = { 0.99870300, 0.98845460, 0.97820620, 0.94512380, 0.88920740, 0.83329100, 0.74075380, 0.64821660, 0.54189260, 0.42178180, 0.30167100};
            Double_t blue[NRGBs]  = { 0.77206100, 0.57616660, 0.38027220, 0.27294620, 0.25418860, 0.23543100, 0.26358420, 0.29173740, 0.27823320, 0.22307160, 0.16791000};
            TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, NCont);

        };
        break;
        case 3:
        {
            const Int_t NRGBs = 6;
            const Int_t NCont = 255;
            Double_t stops[NRGBs] = { 0., 0.166667, 0.333333, 0.5, 0.666667, 0.833333};
            Double_t red[NRGBs]   = { 0.90222000,  0.43315467,  0.14287215,  0.23499124,  0.97291880,  1.00000000};
            Double_t green[NRGBs] = { 0.10180800,  0.33846194,  0.71654974,  0.96760361,  0.93912600,  0.50340849};
            Double_t blue[NRGBs]  = { 0.19830600,  0.99996769,  1.00000000,  0.11563298,  0.20174888,  0.00606769};
            
            TColor::CreateGradientColorTable(NRGBs,stops,red,green,blue,NCont);
        }
        break;
        case 4:
        {
            const Int_t NRGBs = 5;
            const Int_t NCont = 255;

            Double_t stops[NRGBs] = { 0.00, 0.34, 0.61, 0.84, 1.00 };
            Double_t red[NRGBs]   = { 0.00, 0.00, 0.87, 1.00, 0.51 };
            Double_t green[NRGBs] = { 0.00, 0.81, 1.00, 0.20, 0.00 };
            Double_t blue[NRGBs]  = { 0.51, 1.00, 0.12, 0.00, 0.00 };
            TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, NCont);
        };
        break;
        case 5:
        {
            const Int_t NRGBs = 11;
            const Int_t NCont = 255;
            Double_t stops[NRGBs] = { 0., 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.};
            Double_t red[NRGBs]   = { 1.00000000,  1.00000000,  1.00000000,  1.00000000,  0.99175080,  0.97937700,  0.86472300,  0.70518820,  0.45589180,  0.22367580,  0.00000000};
            Double_t green[NRGBs] = { 1.00000000,  0.92934160,  0.84232640,  0.72259760,  0.59019960,  0.45146700,  0.33647640,  0.23501280,  0.16060320,  0.08148000,  0.00000000};
            Double_t blue[NRGBs]  = { 1.00000000,  0.69465640,  0.41882940,  0.20203560,  0.09831576,  0.05113290,  0.18291996,  0.31792300,  0.45935800,  0.30390180,  0.00000000};

            TColor::CreateGradientColorTable(NRGBs,stops,red,green,blue,NCont);
        };
        break;
        case 6:
        {
            const Int_t NRGBs = 11;
            const Int_t NCont = 255;

            Double_t stops[NRGBs] = { 0., 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.};
            Double_t red[NRGBs]   = { 0, 0, 0, 0.259325, 0.564679, 0.806238, 0.956712, 0.999101, 0.928615, 0.8, 0.9};
            Double_t green[NRGBs] = { 0, 0, 0, 0, 0, 0, 0.236495, 0.597902, 0.863921, 0.992114, 0.992114};
            Double_t blue[NRGBs]  = { 0.187381, 0.465151, 0.70265, 0.879316, 0.979855, 0.995562, 0.925077, 0.774503, 0.556876, 0.4, 0.9};
            TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, NCont);
        };
        break;
        case 7:
        {
            const Int_t NRGBs = 11;
            const Int_t NCont = 255;

            Double_t stops[NRGBs] = { 0., 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1. };
            Double_t red[NRGBs]   = { 1., 0.9, 0.8, 0.7, 0.7, 0.5, 0.4, 0.3, 0.2, 0.1, 0. };
            Double_t green[NRGBs] = { 1., 0.9, 0.8, 0.7, 0.7, 0.5, 0.4, 0.3, 0.2, 0.1, 0. };
            Double_t blue[NRGBs]  = { 1., 0.9, 0.8, 0.7, 0.7, 0.5, 0.4, 0.3, 0.2, 0.1, 0. };
            TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, NCont);
        };
        break;
        case 8:
        {
            const Int_t NRGBs = 11;
            const Int_t NCont = 255;

            Double_t stops[NRGBs] = { 0., 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1. };
            Double_t red[NRGBs]   = { 0.122103  , 0.0992373, 0.0867346, 0.0949579, 0.128822, 0.175507, 0.267918, 0.378003, 0.523432, 0.709069, 0.914809 };
            Double_t green[NRGBs] = { 0.00901808, 0.171394 , 0.323342 , 0.454435 , 0.559788, 0.652273, 0.718234, 0.777765, 0.824434, 0.863064, 0.897673 };
            Double_t blue[NRGBs]  = { 0.39826   , 0.465365 , 0.517715 , 0.540552 , 0.540297, 0.528496, 0.496156, 0.461297, 0.421401, 0.385122, 0.350652 };
            TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, NCont);
        };
        default:
        break;
    }
    gStyle->SetNumberContours(NumberOfContours);
}

