#include "TomographyMath.h"

// int InsideSeparatrix(particle part, double eKin, double mass, double eV, int h, double gammaT, double phis)
// {
//   // right now this only handles stationary buckets easy to modify.
//   // Be careful here comparing to dE**2 in if() (What a useless fucking comment.)

//   double SepE = 2*pow( beta( eKin, mass ),2 )*eV*es(eKin,mass)/(slip( eKin, mass, gammaT )*double(h)*2*M_PI);
//   SepE = SepE*(-cos(phis)-cos(part.phi));

//   return ( pow(part.dE,2) < SepE );

// };

// bool compareActionOfCell( int one, int two )
// { 
//   //function to use in sort to compare 
//   return (one > two);
// }

//Binning_____________________________________________________________________________
// int ArrayToVector_Pos(int iBin, int jBin, int iSize){ return ( iBin + (jBin-1)*iSize );  }
// int WhichBin( double point, double binSize ){ return (int)floor( point/binSize + 1); }

//Parameter Calculations _______________________________________________________
double gamma(double eKin, double mass)
{ 
  return 1+eKin/mass; 
}

double beta(double eKin, double mass)
{ 
  return sqrt(eKin*(eKin+2*mass))/(eKin+mass); 
}

double slip(double eKin, double mass, double gammaT)
{ 
  return ( 1/pow(gammaT, 2) - 1/pow( gamma(eKin, mass), 2) ); 
}

double es(double eKin, double mass)
{ 
  return eKin + mass; 
}

double BucketHeight(double eKin, double mass, double gammaT, double eV, int h)
{
  double height = sqrt(eV*es( eKin, mass )/(fabs(slip( eKin, mass, gammaT ))*2*M_PI*double(h) ) );
  height = 2*beta( eKin, mass )*height;

  return height; 
}

//using Machine_Config type
double gamma(Machine_Config config){ return ( 1+(config.eKin)/(config.mass) );  }

double beta(Machine_Config config){  return sqrt( 1 - 1/pow(gamma(config),2) ); }

double slip(Machine_Config config){ return ( 1/pow(config.gammaT, 2) - 1/pow( gamma(config), 2) ); }

double es(Machine_Config config){ return ( config.eKin + config.mass ); }

double BucketHeight(Machine_Config config)
{
  double height = sqrt(config.RF_V*es(config)/( fabs( slip(config) )*2*M_PI*(double)config.h ) );
  height = 2*beta(config)*height;

  return height; 
}

double synchrotronHamiltonian(float deV, float phi, Machine_Config* machine)
{
  //Give dE in eV. Just to be consistent with everything. 
  //This should be a double, probably.
  float dETerm = 0.5* machine->h * slip(*machine) * pow( deV, 2.0f );
  dETerm /= pow( beta(*machine), 2.0 ) * es(*machine);

  float phiTerm = machine->RF_V;
  phiTerm *= cos(phi) - cos(machine->synPhi) + (phi - machine->synPhi)*sin(machine->synPhi);
  phiTerm /= 2*M_PI;

  return dETerm + phiTerm;
}

double SynchrotronPeriod_Turns(Machine_Config config)
{

  //Synch period calculated as 1/nu_s
  //using values available from Machine_Config
  double bet = beta(config);
  double eta = slip(config);

  // Estimate the Synchrotron Period. tau -> 2PI*h/omegaRF
  double numerator   = -2*M_PI*pow(bet,2.0)*(config.eKin);
  double denominator =  eta*(double)config.h*config.RF_V*cos(config.synPhi);

  double synchPeriod = sqrt(numerator/denominator);

  return synchPeriod; //[turns]
}

double SynchrotronPeriod_Seconds(Machine_Config config)
{

  //Synch period calculated as 1/nu_s
  //using values available from Machine_Config
  double bet = beta(config);
  double eta = slip(config);

  // Estimate the Synchrotron Period. tau -> 2PI*h/omegaRF
  double numerator   = -2*M_PI*pow(bet,2.0)*(config.eKin)*(double)config.h;
  double denominator =  eta*pow( config.RF_f, 2 )*config.RF_V*cos(config.synPhi);

  double synchPeriod = sqrt(numerator/denominator);

  return synchPeriod; //[seconds]
}



