#include "emittanceAnalysis.h"

emittanceAnalysis::emittanceAnalysis(complete_data_header_type* head, dataObject* data)
{
	thisHeader = (*head);
	thisData   = data;

	actionOrderedArray.clear();
	contourLimits.clear();
	emittanceCurves.clear();
	
}

emittanceAnalysis::~emittanceAnalysis()
{
	
}

void emittanceAnalysis::orderImageByAction(int numberContours)
{
	profile_data_header_type head = thisHeader.dataAcquisition;
	Tomography_Config tomoHead    = thisHeader.tomography;
	int sizeX = thisData->getImageWidth();
	int sizeY = thisData->getImageHeight();

	double xCenter = (double)sizeX / (double)2.0;
	double yCenter = (double)sizeY / (double)2.0; 
	int arraySize = sizeX * sizeY;

	std::vector< int > allCells;
	for(int cell = 0; cell < arraySize; ++cell) allCells.push_back( cell );	
	
	std::vector< float > contours;

	//Create the contour values to use.
	double BucketHeight_eV = BucketHeight(thisHeader.machine);
	double contourStep = 2.0/(double)numberContours;
	contourValues.clear();
	contourLimits.clear();
	contourValues.push_back(0.0);
	contourLimits.push_back(0);
	for(int i = 1; i <= numberContours; ++i)
	{
		contours.push_back( i*contourStep );
		contourValues.push_back( contours[i-1]*BucketHeight_eV/2.0); // I don't think this is right. 
		contours[i-1] -= 1.0;
	}

	double phiStep = 2*M_PI / (double)sizeX;
	std::vector<int> toDeleteThisPass;

	actionOrderedArray.clear();
	std::vector<int>::iterator firstCell;
	//You'd be much better off going through each cell once 
	//and looping over the contour values. 
	//This method produces no duplicates though, which is nice. 
	int  thisContourEnd = 0;

	for( int whichContour = 0; whichContour < contours.size(); ++whichContour )
	{	
		toDeleteThisPass.clear();
		double thisContour = contours[whichContour];
		firstCell = allCells.begin();
		for(int i = 0; i < allCells.size(); ++i)
		{	
			int thisCell = *(firstCell+i);
			int x = thisCell % sizeX;
			int y = (thisCell - x)/sizeX;
			double actionCurve = sqrt( thisContour - cos( (double)x * phiStep) );

			if( actionCurve > 0) 
			//This just skips this part if newAction = nan w/o having to compare floats
			{
				actionCurve *= yCenter/sqrt(2.0);;
				int actionUpperLimit = (int)ceil(  actionCurve + yCenter + 0.5 );
				int actionLowerLimit = (int)floor(-actionCurve + yCenter - 0.5 ); 
				
				if( y <= actionUpperLimit && y >= actionLowerLimit )
				{
					actionOrderedArray.push_back( thisCell );
					toDeleteThisPass.push_back( i );
					++thisContourEnd;
				}
			}
		}
		//Delete the cells that have been accounted for
		for(int i = 0; i < toDeleteThisPass.size(); ++i) allCells.erase( firstCell + toDeleteThisPass[i] - i );
		contourLimits.push_back(thisContourEnd);
		
	}
}

void emittanceAnalysis::calculateEmittanceContour(int numberContours)
{
	hardPrep();
	orderImageByAction(numberContours);
	profile_data_header_type config = thisData->getDataAcquisition();
	std::vector<int>::iterator actionStart = actionOrderedArray.begin();
	
	emittanceCurves.resize( config.num_bunches );

	int sizeX = thisData->getImageWidth();
	int sizeY = thisData->getImageHeight();
	int totalCells = sizeX * sizeY;
	
	for(int b = 0; b < config.num_bunches; ++b)
	{	
		int thisLimit = 0; 
		std::vector<float>::iterator imageStart = (*(thisData->getImageVec(b))).begin();
		double norm  = 0;
		for(int cell = 0; cell < totalCells; ++cell )
		{
			norm += *( imageStart + cell );
		}
		while( thisLimit < contourLimits.size() )
		{
			double total = 0;
			
			for(int cell = 0; cell < contourLimits[ thisLimit ]; ++cell )
			{
				total += *(imageStart + *(actionStart + cell) );
			}
			++thisLimit;
			emittanceCurves[b].push_back( total/norm );
		}
	}
}

void emittanceAnalysis::writeActionOrderedArrayToFile(const char* filename)
{
	std::ofstream outfile;
	outfile.open( filename, std::ios::out|std::ios::binary );

	outfile.write( (char *)&actionOrderedArray[0], sizeof(int)*actionOrderedArray.size() );
	
	outfile.close();
}

void emittanceAnalysis::writeEmittanceToFile( const char* filename )
{
	std::ofstream outfile;
	outfile.open( filename, std::ios::out|std::ios::binary );
	int size = contourValues.size();

	outfile.write( (char *)&size, sizeof(int) );
	outfile.write( (char *)&contourValues[0], sizeof(double)*contourValues.size() );

	for(int b = 0; b < emittanceCurves.size(); ++b )
	{
		outfile.write( (char *)&emittanceCurves[b][0], sizeof(double)*emittanceCurves[b].size() );
	}
	
	outfile.close();
}


void emittanceAnalysis::hardPrep()
{
	emittanceCurves.clear();
	actionOrderedArray.clear();
	contourValues.clear();
	contourLimits.clear();
}




