#include "injectionAnalysis.h"

injectionAnalysis::injectionAnalysis(complete_data_header_type* head, dataObject* data)
{
	thisHeader = (*head);
	thisData   = data;

	//Moment stuff. This should be handled better.
	//Have int upToOrder; then
	//std::vector< float > orderAmpI; //Inphase for m=0, m=1, m=2, ..., m=upToOrder for each bunch.
	//std::vector< float > orderAmpQ; //Quadrat for m=0, m=1, m=2, ..., m=upToOrder for each bunch.

	imageFirstMoment_Amp.clear();
	imageFirstMoment_Phi.clear();
	imageFirstMoment_FFT.clear();

	iamgeSecondMoment_Amp.clear();
	iamgeSecondMoment_Phi.clear();
	imageSecondMoment_FFT.clear();

	profileFirstMoment_Amp.clear();
	profileFirstMoment_Phi.clear();
	profileFirstMoment_FFT.clear();

	profileSecondMoment_Amp.clear();
	profileSecondMoment_Phi.clear();
	profileSecondMoment_FFT.clear();

	//Emittance stuff
	actionOrderedArray.clear();
	contourLimits.clear();
	emittanceCurves.clear();

	//More Analysis goes here...
	
}

injectionAnalysis::~injectionAnalysis()
{
	
}

void injectionAnalysis::imageDipoleMoments()
{
	int num_bunches = ( thisData->getDataAcquisition() ).num_bunches;

	imageFirstMoment_Amp.resize( num_bunches );
	imageFirstMoment_Phi.resize( num_bunches );
	imageFirstMoment_FFT.resize( num_bunches );

	profile_data_header_type head     = thisData->getDataAcquisition();
	profile_data_header_type& headRef = head;

	for(int bunch = 0; bunch < num_bunches; ++bunch )
	{
		GetReconDipMom( *(thisData->getDataVec( bunch )), headRef, &imageFirstMoment_Amp[bunch], &imageFirstMoment_Phi[bunch], thisData->getMachine() );//Probably will need the vector by reference. 
	}

	//FFT goes here. Not sure if this is correct. What exactly do we want?
	//Input firstImageMoment_Amp, firstImageMoment_Phi, get out firstImageMoment_FFT
	std::vector< double >& amp = imageFirstMoment_Amp;
	std::vector< double >& fft = imageFirstMoment_FFT;
	getArrayFFT( amp, headRef, fft );
}

void injectionAnalysis::calculateProfileMoment(int order)
{
	

}

void injectionAnalysis::orderImageByAction(int numberContours)
{
	profile_data_header_type head = thisHeader.dataAcquisition;
	int sizeX = head.ptsperprofile;
	int sizeY = sizeX;
	double xCenter = (double)sizeX / (double)2.0;
	double yCenter = (double)sizeY / (double)2.0; 
	int arraySize = sizeX * sizeY;

	std::vector< int > allCells;
	for(int cell = 0; cell < arraySize; ++cell) allCells.push_back( cell );	
	
	std::vector< float > contours;

	//Create the contour values to use.
	double BucketHeight_eV = BucketHeight(thisHeader.machine);
	double contourStep = 2.0/(double)numberContours;
	contourValues.clear();
	contourLimits.clear();
	contourValues.push_back(0.0);
	contourLimits.push_back(0);
	for(int i = 1; i <= numberContours; ++i)
	{
		contours.push_back( i*contourStep );
		contourValues.push_back( contours[i-1]*BucketHeight_eV/2.0); // I don't think this is right. 
		contours[i-1] -= 1.0;
	}

	double phiStep = 2*M_PI / (double)sizeX;
	std::vector<int> toDeleteThisPass;

	actionOrderedArray.clear();
	std::vector<int>::iterator firstCell;
	//You'd be much better off going through each cell once 
	//and looping over the contour values. 
	//This method produces no duplicates though, which is nice. 
	int  thisContourEnd = 0;

	for( int whichContour = 0; whichContour < contours.size(); ++whichContour )
	{	
		toDeleteThisPass.clear();
		double thisContour = contours[whichContour];
		firstCell = allCells.begin();
		for(int i = 0; i < allCells.size(); ++i)
		{	
			int thisCell = *(firstCell+i);
			int x = thisCell % sizeX;
			int y = (thisCell - x)/sizeX;
			double actionCurve = sqrt( thisContour - cos( (double)x * phiStep) );

			if( actionCurve > 0) 
			//This just skips this part if newAction = nan w/o having to compare floats
			{
				actionCurve *= yCenter/sqrt(2.0);;
				int actionUpperLimit = (int)ceil(  actionCurve + yCenter + 0.5 );
				int actionLowerLimit = (int)floor(-actionCurve + yCenter - 0.5 ); 
				
				if( y <= actionUpperLimit && y >= actionLowerLimit )
				{
					actionOrderedArray.push_back( thisCell );
					toDeleteThisPass.push_back( i );
					++thisContourEnd;
				}
			}
		}
		//Delete the cells that have been accounted for
		for(int i = 0; i < toDeleteThisPass.size(); ++i) allCells.erase( firstCell + toDeleteThisPass[i] - i );
		contourLimits.push_back(thisContourEnd);
		
	}
}

void injectionAnalysis::calculateEmittanceContour(int numberContours)
{
	hardPrep();
	orderImageByAction(numberContours);
	profile_data_header_type config = thisData->getDataAcquisition();

	std::vector<int>::iterator actionStart = actionOrderedArray.begin();
	
	emittanceCurves.resize( config.num_bunches );
	
	for(int b = 0; b < config.num_bunches; ++b)
	{	
		int thisLimit = 0; 
		std::vector<float>::iterator imageStart = (*(thisData->getImageVec(b))).begin();
		double norm  = 0;
		for(int cell = 0; cell < config.ptsperprofile*config.ptsperprofile; ++cell )
		{
			norm += *( imageStart + cell );
		}
		while( thisLimit < contourLimits.size() )
		{
			double total = 0;
			
			for(int cell = 0; cell < contourLimits[ thisLimit ]; ++cell )
			{
				total += *(imageStart + *(actionStart + cell) );
			}
			++thisLimit;
			emittanceCurves[b].push_back( total/norm );
		}
	}
}

void injectionAnalysis::writeImageMomentToFile(const char* filename)
{
	std::ofstream outfile;
	outfile.open( filename, std::ios::out|std::ios::binary );

	outfile.write( (char *)&imageFirstMoment_Amp[0], sizeof(double)*imageFirstMoment_Amp.size() );
	outfile.write( (char *)&imageFirstMoment_Phi[0], sizeof(double)*imageFirstMoment_Phi.size() );
	outfile.write( (char *)&imageFirstMoment_FFT[0], sizeof(double)*imageFirstMoment_FFT.size() );

	outfile.close();
}

void injectionAnalysis::writeActionOrderedArrayToFile(const char* filename)
{
	std::ofstream outfile;
	outfile.open( filename, std::ios::out|std::ios::binary );

	outfile.write( (char *)&actionOrderedArray[0], sizeof(int)*actionOrderedArray.size() );
	
	outfile.close();
}

void injectionAnalysis::writeEmittanceToFile( const char* filename )
{
	std::ofstream outfile;
	outfile.open( filename, std::ios::out|std::ios::binary );
	int size = contourValues.size();

	outfile.write( (char *)&size, sizeof(int) );
	outfile.write( (char *)&contourValues[0], sizeof(double)*contourValues.size() );

	for(int b = 0; b < emittanceCurves.size(); ++b )
	{
		outfile.write( (char *)&emittanceCurves[b][0], sizeof(double)*emittanceCurves[b].size() );
	}
	
	outfile.close();
}


void injectionAnalysis::hardPrep()
{
	imageFirstMoment_Amp.clear();
	imageFirstMoment_Phi.clear();
	imageFirstMoment_FFT.clear();

	iamgeSecondMoment_Amp.clear();
	iamgeSecondMoment_Phi.clear();
	imageSecondMoment_FFT.clear();

	profileFirstMoment_Amp.clear();
	profileFirstMoment_Phi.clear();
	profileFirstMoment_FFT.clear();

	profileSecondMoment_Amp.clear();
	profileSecondMoment_Phi.clear();
	profileSecondMoment_FFT.clear();

	emittanceCurves.clear();
	actionOrderedArray.clear();
	contourValues.clear();
	contourLimits.clear();
}




