#ifndef _INJECTIONANALYSIS_H
#define	_INJECTIONANALYSIS_H
#include <vector>
#define _USE_MATH_DEFINES
#include <math.h>
#include <algorithm>

#include "dataObject.h"
#include "TomographyMath.h"
#include "TomographyAnalysis.h"

class dataObject;

class injectionAnalysis
{	
	//This should be passed a data object and check if the
	//parameters are the same. It doesn't need to own a pointer
	//to a specific dataObject. 
	private:
		dataObject* thisData;
		complete_data_header_type thisHeader; 

		//Moment stuff. This should be handled better. Maybe have:
		//int upToOrder; then
		//std::vector< float > orderAmpI; //Inphase for m=0, m=1, m=2, ..., m=upToOrder for each bunch.
		//std::vector< float > orderAmpQ; //Quadrat for m=0, m=1, m=2, ..., m=upToOrder for each bunch.
		std::vector< double > imageFirstMoment_Amp;
		std::vector< double > imageFirstMoment_Phi;
		std::vector< double > imageFirstMoment_FFT;

		std::vector< double > iamgeSecondMoment_Amp;
		std::vector< double > iamgeSecondMoment_Phi;
		std::vector< double > imageSecondMoment_FFT;

		std::vector< double > profileFirstMoment_Amp;
		std::vector< double > profileFirstMoment_Phi;
		std::vector< double > profileFirstMoment_FFT;

		std::vector< double > profileSecondMoment_Amp;
		std::vector< double > profileSecondMoment_Phi;
		std::vector< double > profileSecondMoment_FFT;

		std::vector< std::vector< double > > emittanceCurves;
		std::vector< int >    actionOrderedArray;
		std::vector< int >    contourLimits; 
		std::vector< double > contourValues;

		void hardPrep();
		
	public:
		injectionAnalysis(complete_data_header_type *head, dataObject* data);
		~injectionAnalysis();

		void imageDipoleMoments();
		void calculateProfileMoment(int order);

		void orderImageByAction(int numberContours=10);
		void fullAnalysis();
		void calculateEmittanceContour(int numberContours);

		//I/O - maybe change these to take ofstreams 
		void writeImageMomentToFile( const char* filename );
		void writeActionOrderedArrayToFile( const char* filename );
		void writeEmittanceToFile( const char* filename );

		//Getters
		std::vector<double>* getEmittanceCurves(int bunch){ return &emittanceCurves[bunch]; }
		std::vector<double>* getContourValues(){ return &contourValues; }
		std::vector<double>* getImageMomentAmplitudes(const int order){ return &imageFirstMoment_Amp; };
		std::vector<double>* getImageMomentPhase(const int order){ return &imageFirstMoment_Phi; };
};


#endif

