#ifndef _momentAnalysis_H
#define	_momentAnalysis_H
#include <vector>
#define _USE_MATH_DEFINES
#include <math.h>
#include <algorithm>
#include <TH2.h>
#include <TCanvas.h>

#include "dataObject.h"
#include "TomographyMath.h"
#include "TomographyAnalysis.h"


class momentAnalysis
{
	private:

		int currentOrder;
		dataObject *assocData;

		std::vector< TH2D* > normalMatrices; //inphase components
		std::vector< TH2D* > skewedMatrices; //quadrature components

		//These are given as ( {bunch1 0,1,2...currentOrder},{bunch2 0,1,2...curentOrder}... )
		std::vector< double > inphase; //These are the computed strengths. ( momentMatrix->Multiply(Image) )->Integral("width")
		std::vector< double > quad;    //These are the computed strengths. ( momentMatrix->Multiply(Image) )->Integral("width")

		int factorial( int x );
		double mChooseK( int m, int k );

		void generateOrderM( int m, TH2D* hist, int normORskew, double maxRad, double yScale, double centerX=0.5, double centerY=0.5 );
		double calculateYScale( double dPhiMax, int nx, int ny );

	public:

		momentAnalysis();
		~momentAnalysis();

		void setData( dataObject* data ){ assocData = data; };

		void generateMomentMatrices( int upToOrder, int coordinates=kCartesian );
		void calculateModeContent();

		double getInphase(int order, int bunch);
		double getQuad(int order, int bunch);

		void extractPhases( std::vector<double> phaseStorage );

		bool hasData(){ return assocData != NULL; };
		bool hasNoData(){ return assocData == NULL; };

		void printRawMomentEllipsesToFile();

		TH2D* getMthOrderMatrix(int order, int orientation=kNormal); 

		enum coord
		{
			kCartesian,
			kPolar
		};

		enum orientation
		{
			kNormal,
			kSkewed
		};

};

#endif 
