#include "ppBunchMidFinder.h"

ppBunchMidFinder::ppBunchMidFinder(float bucketLengthIN,float bunchChargeFullIN, float bunchChargePartialIN, int ptsPerProfileIN)
{
    bucketLength       = bucketLengthIN;//ctor
    bunchChargeFull    = bunchChargeFullIN;
    bunchChargePartial = bunchChargePartialIN;
    ptsPerProfile      = ptsPerProfileIN;
}

ppBunchMidFinder::~ppBunchMidFinder()
{
    delete sumFramesInterp;//dtor
}

bool ppBunchMidFinder::getBunchMids(const std::vector<long int>& sumFrames, getBunchMidMethod method,std::vector<float>& bunchMids)
{
    quickOffset = quickGuessOffset(sumFrames);
    maxNumBunches = (int)floor( ((float)sumFrames.size() - quickOffset )/bucketLength);
    #ifdef _PREPOCESSER_VERBOSE
        std::cout << std::endl;
        std::cout << "\tMax Num Bunches from offset" << maxNumBunches << std::endl;
        std::cout << std::endl;
    #endif


    //std::cout << "maxNumBunches "<< maxNumBunches << std::endl;
    getQuickProfilePositions();
    /// need a background level -
    long int background = getBackGround(sumFrames);
    //std::cout << "background "<< background << std::endl;
    getProfileOccupancy(sumFrames,background);
    /// To better find the bucket positions we take (different possible) averages of moments,
    /// Assuming speed isn't an issue (generally it isnt) we can interpolate sumFrames to help with this
    interpSumFrames(sumFrames, ROOT::Math::Interpolation::kLINEAR); /// Interpolation::XXX options CSPLINE, LINEAR, POLYNOMIAL, CSPLINE_PERIODIC, AKIMA, AKIMA_PERIODIC
    interpStep = bucketLength/ptsPerProfile;
    //std::cout << "interpStep " << interpStep << std::endl;
    float offset;
    getProfileWeights(method);

    double optOffset = findOptOffset(quickOffset);
    #ifdef _PREPOCESSER_VERBOSE
        std::cout << std::endl;
        std::cout << "\toptOffset =" << optOffset << std::endl;
        std::cout << "\t" << std::endl;
        std::cout << std::endl;
    #endif
    //std::cout << "optOffset " << optOffset << std::endl;
    writeBunchMids(bunchMids,(float)optOffset);

    return false;
}


double ppBunchMidFinder::findOptOffset(double offsetIN)
{
    double iOffset = offsetIN-5;
    if(iOffset < 0)
    {
        iOffset = 0;
    }
    #ifdef _PREPOCESSER_VERBOSE
        std::cout << std::endl;
        std::cout << "\tfindOptOffset inital iOffset " << iOffset << std::endl;
        std::cout << std::endl;
    #endif
    double guess1,guess2;
    guess1 = calcMean1stMom(iOffset);
    guess2 = calcMean1stMom(iOffset+1);
    //std::cout << "guess1 " << guess1 << std::endl;
    //std::cout << "guess2 " << guess2 << std::endl;
    int posTest;
    posTest = Bckt_Offset_Test(guess1, guess2);
    while(posTest!=0) /// this test the gradient and moves closer to the desired answer of a HBL...
    {
        if(posTest==1)
        {
            iOffset += 1;
            guess1 = guess2;
            guess2 = calcMean1stMom(iOffset);
            posTest = Bckt_Offset_Test(guess1,guess2);
            //std::cout << "posTest END  "<< posTest << "  " << iOffset << "  " << guess1 << "  " << guess2 <<std::endl;
        }else{

            if(posTest==-1)
            {
                  iOffset -= 1;
                  guess2 = guess1;
                  guess1 = calcMean1stMom(iOffset);
                  posTest = Bckt_Offset_Test(guess1,guess2);
                  //std::cout << "posTest END  "<< posTest << "  " << iOffset << "  " << guess1 << "  " << guess2 <<std::endl;
            }else{

                std::cout << "ppBunchMidFinder::findOptOffset-ERROR";
            }
        }
    }
    return  linInterp(iOffset,guess1,iOffset+1,guess2,(double)bucketLength/2.0);
}

int ppBunchMidFinder::Bckt_Offset_Test(double mean1stMom1, double mean1stMom2)
{
    double halfBcktLen = (double)bucketLength/2.0;
    if(mean1stMom2 > mean1stMom1) /// +ve grad, forward 1
    {
      return 1;
    }else{

        if(mean1stMom1<halfBcktLen && mean1stMom2<halfBcktLen) /// -ve gradient & both less than halfBcktLen, back 1
        {
            return -1;
        }else{

            if( mean1stMom1 > (double)halfBcktLen && mean1stMom2 > (double)halfBcktLen) /// -ve gradient & both more than halfBcktLen, forward 1
            {
                return 1;
            }else{
                if( (mean1stMom1>mean1stMom2 && (mean1stMom1>(double)halfBcktLen)&&(mean1stMom2<(double)halfBcktLen)) )
                {
                    return 0;
                }else{
                    return 2; /// ERROR
                }
            }
        }
    }
}
void ppBunchMidFinder::getProfileWeights( getBunchMidMethod method) // create vector to weight each moment i.e. the different methods
{

    int count;
    profileWeights = std::vector<bool > (profileOccupancies.size());

    switch(method)
    {
        case quick:

            for(int i = 0 ; i< profileWeights.size() ; ++i)
            {
                switch(profileOccupancies[i])
                {
                    case empty:
                        profileWeights[i]=0;
                        break;
                    case partial:
                        profileWeights[i]=1;
                        break;
                    case full:
                        profileWeights[i]=1;
                }
            }
        break ;

        case allbunches:

            for(int i = 0 ; i< profileWeights.size() ; ++i)
            {
                switch(profileOccupancies[i])
                {
                    case empty:
                        profileWeights[i]=0;
                    break;
                    case partial:
                        profileWeights[i]=1;
                    break;
                    case full:
                        profileWeights[i]=1;
                }
            }
        break ;


        case first:

            count = 0;
            for(int i = 0 ; i < profileWeights.size() ; ++i)
            {
                if( profileOccupancies[i] == empty)
                {
                    profileWeights[i]=0;
                    ++count;
                }
                else
                {
                    profileWeights[count]=1;
                    for(int j = count+1 ; j  < profileWeights.size() ; ++j)
                    {
                        profileWeights[j]=0;
                    }
                    break;

                }
            }
        break ;

        case firstfull:

            count = 0;
            for(int i = 0 ; i < profileWeights.size() ; ++i)
            {
                if( profileOccupancies[i] == empty || profileOccupancies[i] == partial)
                {
                    profileWeights[i]=0;
                    ++count;
                }
                else
                {
                    profileWeights[count] = 1;
                    for(int j = count+1 ; j  < profileWeights.size() ; ++j)
                    {
                        profileWeights[j]=0;
                    }
                    break;

                }
            }
        break ;

        case fullbunches:
            for(int i = 0 ; i< profileWeights.size() ; ++i)
            {
                switch(profileOccupancies[i])
                {
                    case empty:
                        profileWeights[i]=0;
                    break;
                    case partial:
                        profileWeights[i]=0;
                    break;
                    case full:
                        profileWeights[i]=1;
                }
            }
        break;

        default:
            for(int i = 0 ; i< profileWeights.size() ; ++i)
            {
                switch(profileOccupancies[i])
                {
                    case empty:
                        profileWeights[i]=0;
                    break;
                    case partial:
                        profileWeights[i]=1;
                    break;
                    case full:
                        profileWeights[i]=1;
                }
            }
            break;
    };
    #ifdef _PREPOCESSER_VERBOSE
        std::cout << std::endl;
        std::cout << "\tProfile Weight List =" << std::endl;
        std::cout << "\t" << std::endl;
        for(int i = 0 ; i < profileWeights.size() ; ++i)
        {
             std::cout << profileWeights[i] << " ";
        }
        std::cout << std::endl;
        std::cout << std::endl;
    #endif

    //for(int i =0 ; i<profileOccupancies.size() ; ++i )
    //std::cout << "profileOccupancy  " << profileOccupancies[i] << "  "  << profileWeights[i] << std::endl;
}
void ppBunchMidFinder::writeBunchMids(std::vector<float>& bunchMids,float offset)
{
    //std::cout << "offset[i] " << offset << std::endl;
    int count;
    count = 0;
    for(int i = 0 ; i < quickProfileStartPos.size(); ++i)
    {
        //std::cout << "profileOccupancies[i] " << profileOccupancies[i] << std::endl;
        if( profileOccupancies[i] != empty)
        {
            bunchMids[count]=offset+(i*bucketLength+bucketLength/2.0);
            ++count;
        }
    }
    bunchMids.resize(count);
    #ifdef _PREPOCESSER_VERBOSE
        std::cout << std::endl;
        std::cout << "\tBunch Mids ListList =" << std::endl;
        std::cout << "\t" << std::endl;
        for(int i = 0 ; i < bunchMids.size() ; ++i)
        {
             std::cout << bunchMids[i] << " ";
        }
        std::cout << std::endl;
        std::cout << std::endl;
    #endif

    //for(int i=0;i<2;++i)
    //std::cout << "bunchMids[i] " << bunchMids[i] << std::endl;

}
void ppBunchMidFinder::interpSumFrames(const std::vector<long int>& sumFrames, ROOT::Math::Interpolation::Type interpType)
{ /// Interpolation::XXX options CSPLINE, LINEAR, POLYNOMIAL, CSPLINE_PERIODIC, AKIMA, AKIMA_PERIODIC
  /// http://root.cern.ch/drupal/content/function-interpolation
    int ni = sumFrames.size();
    double *xi = new double[ni]();  /// have to allocate memory at run-time..
    double *yi = new double[ni]();
    sumFramesInterp = new ROOT::Math::Interpolator(ni, interpType);//
    for ( int i = 0; i <ni; ++i )
    {
            xi[i] = (double) i;
            yi[i] = (double) sumFrames[i];
    }
    sumFramesInterp -> SetData(ni,xi, yi);
    interpMax       =  sumFrames.size();
    delete[] xi;
    delete[] yi;

    /*double tempd;
    for(int i=0;i<10;++i)
    {
    tempd= sumFramesInterp->Eval(i+0.3);
    std::cout << "tempd " << tempd << std::endl;
    }*/
}

double ppBunchMidFinder::calc1stMom(double start)
{
    double datPos = 0.0;       /// position to get data
    double dat    = 0.0;       /// interpolated data value
    double sumDatDatPos = 0.0; /// dat  * datPos integrated
    double sumDat       = 0.0; /// dat integrated
    for(int i = 0; i < ptsPerProfile ; ++i)
    {
        datPos        = i*interpStep  + 0.5*interpStep;  /// (assume) we are giving the value at the centre of a bin
        dat           = sumFramesInterp -> Eval( start + datPos);
        sumDatDatPos += datPos*dat ;
        sumDat       += dat;

    }
    //std::cout << "sumDatDatPos" << sumDatDatPos << std::endl;
    //std::cout << "sumDat" << sumDat << std::endl;
    return sumDatDatPos/sumDat;
}

double ppBunchMidFinder::calcMean1stMom(double offsetIN)
{
    double sumMom,offset,mom;
    int count;
    count  = 0;
    sumMom = 0;
    for(int i = 0 ; i < profileWeights.size() ; ++i) /// iterate over profiles
    {
        offset =  (double)offsetIN +  (double)i* (double)bucketLength;
        if(offset + (double)bucketLength > (double)interpMax) break;
        //std::cout << "offset " << offset << std::endl;
        mom = profileWeights[i]*calc1stMom(offset);
        //std::cout << "mom= " << mom << std::endl;
        sumMom += mom;
        //std::cout << "sumMom " << sumMom << std::endl;
    }
    count = (int)accumulate(profileWeights.begin(),profileWeights.end(), 0.0);
    return  sumMom/(double)count;
}

double ppBunchMidFinder::linInterp(double x1, double y1, double x2, double y2, double y)
{
  return  (x2*y - x1 * y + x1* y2 - x2*y1)/( y2 - y1);
}
/// _______         __        __         _______         __   __             __
///|       |.--.--.|__|.----.|  |--.    |   |   |.-----.|  |_|  |--.-----.--|  |
///|   -  _||  |  ||  ||  __||    <     |       ||  -__||   _|     |  _  |  _  |
///|_______||_____||__||____||__|__|    |__|_|__||_____||____|__|__|_____|_____|
void ppBunchMidFinder::getQuickProfilePositions() /// gets the start and stop position of each profile, need to do this to assign full, partial or no beam
{
    quickProfileStartPos = std::vector<int>(maxNumBunches);
    quickProfileEndPos   = std::vector<int>(maxNumBunches);
    //std::cout << "bucketLength  " << bucketLength << std::endl;
    for(int k = 0 ; k < maxNumBunches - 1 ; ++k )
    {
        quickProfileStartPos[k]=roundPos(quickOffset + k*bucketLength);
        quickProfileEndPos[k]=roundPos(quickOffset + (k+1)*bucketLength);
    }
    #ifdef _PREPOCESSER_VERBOSE
        std::cout << std::endl;
        std::cout << "\tQuick Bunch mids = " << std::endl;
        std::cout << "\t" << std::endl;
        for(int i = 0 ; i < quickProfileEndPos.size() ; ++i)
        {
             std::cout << quickOffset+bucketLength /2.0 + i*bucketLength << " ";
        }
        std::cout << std::endl;
        std::cout << "\tMax Num Bunches from offset = " << maxNumBunches << std::endl;
        std::cout << std::endl;
    #endif

}

void ppBunchMidFinder::getProfileOccupancy(const std::vector<long int>& sumFrames,long int background)
{
    int i;
    long int profTotminCommonestMax,temp1,temp2;
    float profChargeNorm;

    std::vector<long int> profCharge (quickProfileStartPos.size() );
    for(i = 0; i<profCharge.size();++i)
    {
      temp1=(long int)accumulate(sumFrames.begin()+quickProfileStartPos[i], sumFrames.begin() + quickProfileEndPos[i]+1, 0.0);
      temp2=((quickProfileEndPos[i] - quickProfileStartPos[i]) * background);
      //std::cout << "temp1  " << temp1 << std::endl;
      //std::cout << "temp2  " << temp2 << std::endl;
      profCharge[i] =temp1-temp2;
      //accumulate(sumFrames.begin()+quickProfileStartPos[i], sumFrames.begin() + quickProfileEndPos[i], 0.0)
      //- ((quickProfileEndPos[i] - quickProfileStartPos[i]+1) * background);
      //std::cout << "profCharge  " << profCharge[i] << std::endl;
    }
    profTotminCommonestMax = *max_element(profCharge.begin(), profCharge.end()); // Normalise profile charge to max for test
    profileOccupancies = std::vector<ppBunchMidFinder::profileOccupancy>(profCharge.size());

    for(i=0;i<profCharge.size();++i) /// test to see if there is a bunch or not
    {
        profChargeNorm = (float)profCharge[i]/ (  (float)profTotminCommonestMax);
        //std::cout << "profChargeNorm " << profChargeNorm << std::endl;
        if(profChargeNorm < bunchChargePartial)
        {
           profileOccupancies[i] = ppBunchMidFinder::empty;
        }
        else if(profChargeNorm > bunchChargeFull)
        {
            profileOccupancies[i] = ppBunchMidFinder::full;
        }
        else
        {
            profileOccupancies[i] = ppBunchMidFinder::partial;
        }

    }
    #ifdef _PREPOCESSER_VERBOSE
        std::cout << std::endl;
        std::cout << "\tProffile Occupancy List =" << std::endl;
        std::cout << "\t" << std::endl;
        for(int i = 0 ; i < profileOccupancies.size() ; ++i)
        {
             std::cout << profileOccupancies[i] << " ";
        }
        std::cout << std::endl;
        std::cout << std::endl;
    #endif
}

float ppBunchMidFinder::quickGuessOffset(const std::vector<long int>& sumFrames) /// finds position of max in sumFrames, then returns remainder when divided by bucketlength, including data range offset...
{
    numPoints = sumFrames.size();
    int i,maxpos = 0;
    float ans;
    unsigned long int maxval = 0;
    for(int i = (int)floor(bucketLength/2.0) + 1; i < sumFrames.size(); ++i)
    {
        if(sumFrames[i] > maxval)
        {
            maxpos = i;
            maxval = sumFrames[i];
        }
    }/// now check to see if there is more than one max value
    int maxValDuplicates = -1;
    for( int i = 0 ; i < sumFrames.size(); ++i )
    {
        if(sumFrames[i] == maxval)
        {
            maxValDuplicates += 1;
        }
    }

    float a = fmodf((float)maxpos - bucketLength /2.0 , bucketLength);
    #ifdef _PREPOCESSER_VERBOSE
        std::cout << "\tSumFrames Max = " << maxval << ", at postion " << maxpos << std::endl;
        std::cout << "\tThere are " << maxValDuplicates << " duplicates of this position, (higher numbers suggest data saturation)."<< std::endl;
        std::cout << "\tQuick Guess bucket offset Position = " << a << std::endl;
        std::cout << std::endl;
    #endif
    return a;
}

long int ppBunchMidFinder::getBackGround(const std::vector<long int>& sumFrames) ///
{
    long int ans = sumFramesCommonest(sumFrames);
    return ans;
}

/// Hash table implementation of finding commonest element of vector of long ints
typedef std::pair<long int,long int> mode_pair;
struct ppBunchMidFinder::mode_predicate
{
    bool operator()(mode_pair const& lhs, mode_pair const& rhs)
    {
        return lhs.second < rhs.second;
    }
};
long int ppBunchMidFinder::sumFramesCommonest(const std::vector<long int>& sumFrames)
{
    std::map<long int, long int> mode_map;
    for (int n = 0; n < numPoints ; n++)
    {
        mode_map[sumFrames[n]]++;
    }
    mode_predicate mp;
    return std::max_element(mode_map.begin(), mode_map.end(), mp) -> first;
}

