#ifndef PPBUNCHMIDFINDER_H
#define PPBUNCHMIDFINDER_H

#include <algorithm>
#include <iostream>
#include <numeric>
#include <math.h>
#include <vector>
#include <map>

//#include "ppStruct.h"
#include "TomographyStructs.h"
//#include "TomographyStructDefaults.h"

#include "Math/Interpolator.h"
#include <TObject.h>
//#include "TMath.h"

class ROOT::Math::Interpolator;

class ppBunchMidFinder
{
    public:
        ppBunchMidFinder(float bucketLengthIN,float bunchChargeFullIN, float bunchChargePartialIN, int ptsPerProfileIN);
        //virtual ~ppBunchMidFinder();
        ~ppBunchMidFinder();

        bool getBunchMids(const std::vector<long int>& sumFrames,getBunchMidMethod method,std::vector<float>& bunchMids);


    protected:

    private:

        /// quick method
        float quickGuessOffset(const std::vector<long int>& sumFrames);
        float quickOffset;
        int numPoints;
        int quickGuessOffset();
        void getQuickProfilePositions();
        std::vector<int> quickProfileStartPos;
        std::vector<int> quickProfileEndPos;



        double findOptOffset(double offsetIN);
        int Bckt_Offset_Test(double mean1stMom1, double mean1stMom2);


    /// Interpolator
        void interpSumFrames(const std::vector<long int>& sumFrames, ROOT::Math::Interpolation::Type interpType);
        ROOT::Math::Interpolator * sumFramesInterp;
        float interpStep;
        int interpMax;

    /// moments
        double calc1stMom(double start);
        double calcMean1stMom(double offsetIN);
        double linInterp(double x1, double y1, double x2, double y2, double y);

    /// Occuapancy and different method weights
        void getProfileOccupancy(const std::vector<long int>& sumFrames,long int background);
        enum profileOccupancy {empty,partial,full};
        std::vector<ppBunchMidFinder::profileOccupancy> profileOccupancies;
        std::vector<bool > profileWeights;
        void getProfileWeights( getBunchMidMethod method);

        void writeBunchMids(std::vector<float>& bunchMids,float offset);


        float bucketLength, bunchChargeFull, bunchChargePartial;
        int maxNumBunches,ptsPerProfile;

        /// Maths (ahem)
        /// round some  POSITIVE numbers....
        int roundPos(float in) { return (int)floor(in + 0.5); }
        int roundPos(double in) { return (int)floor(in + 0.5); }
        long int getBackGround(const std::vector<long int>& sumFrames);
        struct mode_predicate;
        long int sumFramesCommonest(const std::vector<long int>& sumFrames);

        //ClassDef(ppBunchMidFinder,0);
};

#endif // PPBUNCHMIDFINDER_H
