#include "ppProfileGenerator.h"
#include <fstream>

ppProfileGenerator::ppProfileGenerator(int requestFrameStart, int requestFrameEnd, int requestBatchStart, int requestBatchEnd, const ppDataObjectInfo * DataObjectInfo)
{
    localFrameStart   = requestFrameStart; //ctor
    localFrameEnd     = requestFrameEnd;   //ctor
    localBatchStart   = requestBatchStart; //ctor
    localBatchEnd     = requestBatchEnd;   //ctor
    localBucketLength = DataObjectInfo -> bucketLength;
    localPtsPerFrame  = DataObjectInfo -> sp.ptsPerFrame;
    localInterval     = DataObjectInfo -> sp.interval; ///changes if we interpolate
    localPPP          = DataObjectInfo -> pps;

    #ifdef _PREPOCESSER_VERBOSE
        std::cout << std::endl;
        std::cout << "\t === ppProfileGenerator === " << std::endl;
        std::cout << "\t" << std::endl;
        std::cout << "\t Constructed with thse parameters"  << std::endl;
        std::cout << std::endl;
        std::cout << "\t FrameStart  \t= "          << localFrameStart   << std::endl;
        std::cout << "\t FrameEnd    \t= "          << localFrameEnd     << std::endl;
        std::cout << "\t Batch Start Pos  \t= "     << localBatchStart   << std::endl;
        std::cout << "\t Batch End Pos  \t= "       << localBatchEnd     << std::endl;
        std::cout << "\t BucketLength  \t= "        << localBucketLength << std::endl;
        std::cout << "\t PtsPerFrame  \t= "         << localPtsPerFrame  << std::endl;
        std::cout << "\t Interval  \t= "            << localInterval     << std::endl;
        std::cout << "\t PPP.profileMethod  \t= "   << localPPP.profileMethod << std::endl;
        std::cout << "\t PPP.ptsPerProfile  \t= "   << localPPP.ptsPerProfile    << std::endl;
        std::cout << std::endl;
    #endif

}

ppProfileGenerator::~ppProfileGenerator()
{
    //dtor
}


bool ppProfileGenerator::getProfiles(std::vector <float> & profiles, std::vector <int> & rawData, std::vector <float> & bunchMids)
{
    getFrameBaseline(rawData);

    switch( localPPP.profileMethod )
    {
        case interp:
            interProf(profiles, rawData, bunchMids);
        break;
        case simple:
            simpleProf(profiles, rawData, bunchMids);
        break;
    };
    return false;
}

bool ppProfileGenerator::simpleProf(std::vector <float> & profiles, std::vector <int> & rawData, std::vector <float> & bunchMids)
{
    localPtsPerProf;
    localnumBunches = bunchMids.size();

    if( abs( (float)localBucketLength - (float)localPPP.ptsPerProfile) > 1.0 )
    {
        localPtsPerProf = (int)floor(localBucketLength);
    }else{
        localPtsPerProf = localPPP.ptsPerProfile;
    }
    localPtsPerProf += (localPtsPerProf % 2 == 0 ) ? 1 : 0; /// forced to be odd, which is done in many places around the code

    int localNumFrames = localFrameEnd - localFrameStart;

    profiles.resize( localnumBunches * localNumFrames * localPtsPerProf );

    int proPos;
    int rawPos;
    int localPtsPerBunch = localPtsPerProf * localNumFrames;

    for(int k = 0; k < localnumBunches; ++k) 		    /// loop over bucket middle position
    {
        for(int j = 0; j < localNumFrames; ++j)// + 1; ++j) 		    /// loop over frame
        //for(int j = localFrameStart; j < localFrameEnd + 1; ++j) 		    /// loop over frame
        {
            for(int i = 0 ; i < localPtsPerProf; ++i)
            {
                proPos = i + (j * localPtsPerProf) + (k * localPtsPerBunch); // pt + frame + bunch
                rawPos = ( (j + localFrameStart) * localPtsPerFrame + roundPos( bunchMids[k] ) - (localPtsPerProf-1)/2 + i );
                profiles[ proPos ] = (float)rawData[ rawPos ];// - baseline[j];
            }
        }
    }
    removeProfileBaseLin(profiles);/// This now my preffered method
    return false;
}

void ppProfileGenerator::getProfHeader(profile_data_header_type * profHeader)
{
    profHeader -> num_bunches    = localnumBunches;
    profHeader -> num_frames     = (localFrameEnd - localFrameStart) + 1;
    profHeader -> ptsperprofile  = localPtsPerProf;
    profHeader -> hscale         = localInterval;
    //
    //profHeader->title          = ;
    //profHeader->time           = ;
    //profHeader->delay          = ;
    //profHeader->turns_per_trig = ;
    //profHeader->time_last_meas = ;
    //profHeader->version        = ;
    //
}

bool ppProfileGenerator::interProf(std::vector <float> & profiles, std::vector <int> & rawData, std::vector <float> & bunchMids)
{
    //Not yet implemented I guess? This is supposed to do some interpolation on profiles?
    /// CORRECT
    return false;
}

void ppProfileGenerator::getFrameBaseline(std::vector<int >& inData)
{
    baseline = std::vector<int> ( (localFrameEnd - localFrameStart)+1);
    getCommonest(inData , baseline);
}

void ppProfileGenerator::removeProfileBaseLin(std::vector<float >& profiles)
{/// In this method we take the mean of two points at the start and end of the profiles, by the separatrix and
 /// and assume there is zero beam there, so this must be the baseline. Do for each frame of each profile separatly

    int totnumprof = profiles.size() / localPtsPerProf; /// This should be an int.
    float start1,start2,end1,end2,baseLine;

    for(int i = 0; i < totnumprof ; ++i ) /// loop over profile start positions
    {
        start1 = profiles[ i * localPtsPerProf +1];
        start2 = profiles[ i * localPtsPerProf +2];
        end1   = profiles[ (i + 1) * localPtsPerProf -1];
        end2   = profiles[ (i + 1) * localPtsPerProf -2];
        baseLine = (start1 + start2 + end1 + end2)/4.0;\

        for(int j = 0 ; j < localPtsPerProf ; ++j )
        {
            profiles[ i * localPtsPerProf +j] = profiles[ i * localPtsPerProf +j] - baseLine;
        }
    }
}


typedef std::pair<int,int> mode_pair;
struct ppProfileGenerator::mode_predicate
{
    bool operator()(mode_pair const& lhs, mode_pair const& rhs)
    {
        return lhs.second < rhs.second;
    }
};

int ppProfileGenerator::Commonest(std::vector<int>::iterator start)
{
    std::map<int,int> mode_map;
    for (int n = 0; n < localPtsPerFrame; n++)
    {
        //mode_map[myPrePocesser->sumFrames[n]]++;
        mode_map[*(start+n)]++;
    }
    mode_predicate mp;
    return std::max_element(mode_map.begin(), mode_map.end(), mp) -> first;
}

void ppProfileGenerator::getCommonest(std::vector<int >& inData, std::vector<int >& outData)
{

    for(int i = localFrameStart ; i < localFrameEnd+1 ; ++i )
    {
        outData[i-localFrameStart] = Commonest(inData.begin()+i*localPtsPerFrame);
    }
}








