#ifndef _MapClass_H_
#define _MapClass_H_
//stdlib includes
#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <time.h>
#include <string>
#define _USE_MATH_DEFINES
#include <math.h>
#include <map>

//NJE includes
#include "TomographyStructs.h"
//#include "TomographyStructDefaults.h"
#include "TomographyConfig.h"
#include "TomographyMath.h"

typedef std::map<int, int> Level1Map;
typedef std::map<int, Level1Map> Level2Map;

class MapClass
{

	public:

		enum mapType
		{
			kStandard,
			kExtended,
			kSparse,
			kSparseSingleUse,
			kSparseExtended,
			kIGOR
		};

		MapClass( const complete_data_header_type thisHeader, const std::string& path, const int type = kStandard, const double center = -1.0 );
		~MapClass();
		
		void printHeader();
		void printDataAcquisition();
		void printTomography();
		void printMachine();
		void dumpMapsToStdOut();

		void setTomography(const Tomography_Config newTomography);
		void setMachine(const Machine_Config newMachine);

		int mapLength; 
		
		std::vector<int> PixV;
		std::vector<int> BinV;
		std::vector<int> ValV;

		complete_data_header_type header;

		std::vector<int> nSubJ  ; //Stores number of pixels contributing to binj.
		std::vector<int> BinTot ; //Stores number of pixels contributing to binj.
		std::vector<int> PixTot ; //Stores number launched from pixel i.

		double mapCenter; 

	private:

		time_t timestamp;

		std::string MapPath;

		bool mapsExist();
		bool mapsSuffice( const complete_data_header_type oldHeader );

		void readInMaps();
		void createMaps(const int type);
		void igorsMaps();
		
		void fillNSubJ();	

		void calculateMapCharge();
		void mapToLocalVectors( const std::vector< Level2Map > &fiMap, const int type);
		void writeMapFiles( const std::vector< Level2Map > &fiMap, const int type);
		void logMap();
};

#endif
