#ifndef _TomographyStructs_H_
#define _TomographyStructs_H_
#include <vector>
/*________________________________________________________/
|
| The following data types are derined in this header:
| Raw data header_____________________
|          trigParam
|          scopeParam
|          DAQStruct
|
| Structs for reconstruction__________
|          Machine_Config
|          Tomography_Config
|          Scope_Config
|          profile_data_header_type
|          Format_Config
|          complete_data_header_type
|
| Ming-Jen's header___________________
|          channel_setting
|          header_type
|
| Various enums
\-------------------------------------------------------*/
#pragma pack(push)
#pragma pack(1)

//Trigger box
typedef struct trigParam  /// Hopefully the second trigger box will be similar to the MI one, if not will add new struct
{
 int   numTraces;
 int   turnsDelay;
 int   beamSynchOffset;
 float trigDelay;       //profile data header type delay
 int   trigEvents[50];  //YIKES! GUI only handles five, might have to fix somethings.
 int   trigState;
 char  padding[512];
} trigParam;

//Scope
typedef struct scopeParam
{
  int   numFrames;    //number of records
  float range;        //vscale
  float level;        //trigger level
  float offset;       //DC offset
  int   channel;      //ok
  float interval;     //hscale
  int   ptsPerFrame;  //raw pts/frame from scope, record length
  char  padding[512];
} scopeParam;

//Tigger Box + Scope - This is the raw data header.
typedef struct DAQStruct
{
  int    version;
  char   title[60];
  char   time[14];
  struct scopeParam DAQScope;
  struct trigParam  DAQTrig;
  char   padding[512];
} DAQStruct;

//This will be deprecated in favor of DAQStruct
typedef struct
{
  char	title[60];	     // File title
  char	time[14];	       // Time and data at data acquisition
  int		delay;		       // Trigger clock event delay (t0=us)
  int		turns_per_trig;	 // Measured # of turns between trigs
  float	time_last_meas;	 // Time of last trigger - t0 (sec)
  float	hscale;		       // Horizontal scaling, ns/count
  short	version;	       // version # to track updates
  int   num_bunches;
  int   num_frames;
  int   ptsperprofile;
} profile_data_header_type;

//Machine Info.
typedef struct Machine_Config
{
  int    h;      // [int]
  double eKin;   // []
  double mass;   // [eV] change this to MeV
  double RF_V;   // [V]
  double RF_f;   // [Hz]
  double gammaT; // Real Number
  double synPhi; // ?rad? check maps
} Machine_Config;

//Tomography Info.
typedef struct Tomography_Config
{
  int    MaxIterations;       // Max Iterations
  int    SqrtPPP;             // Sqrt Particles per pixel
  int    Frames;              // Number of Frames used
  int    ReconstructionFrame; // Where to Reconstruct limited to first frame for now.
  double ConvergenceLimit;    // delta discrepancy turn-turn that signals convergence.
  float  dampingFactor;       // A damping factor only used if algorithm is DFCART. 1 = no damping, 0 = complete.
  int    algorithm;           // Algorithm to be used. use enum in this file
  int    pixelsPhi;           // number of image pixels in phi. For generality these don't nee to be the same as the number of bins
  int    pixelsdE;            // number of image pixels in dE.  For generality these don't nee to be the same as the number of bins
} Tomography_Config;

//Struct combining all info. for an aquisition + reconstruction.
typedef struct complete_data_header_type
{
  profile_data_header_type  dataAcquisition;
  Machine_Config            machine;
  Tomography_Config         tomography;
} complete_data_header_type;

//Used in Ming-Jen's header_type below
typedef struct
{
  short		state;
  short		polarity;
  float		vscale;
  float		voffset;
  float		trace_offset;		// trace time offset in nsec
  char		label[41];
} channel_setting;

//Ming-Jen's header. Used in another Mtn Range program.
typedef struct
{
  char			title[60];	     // File title
  char			time[14];	       // Time and data at data acquisition
  int			  reset;		       // Trigger clock event number
  int			  delay;		       // Trigger clock event delay (t0=us)
  int			  azimuth;	       // Azimuthal delay around ring (ns)
  int			  turns_per_trig;	 // Measured # of turns between trigs
  float			time_last_meas;	 // Time of last trigger - t0 (sec)
  int			  _time_offset[4]; // trace time offset in nsec	, no longer used
  int			  snp_di[4];	     // Snapshot device DIs
  int			  snp_num_dev;	   // Number of snapshot devices
  int			  snp_period;	     // Requested length of plot (ms)
  short			snp_delay;	     // Requested delay after reset (ms)
  int			  rtd_len;	       // Length of each record in bytes
  short			rtd_nrec;	       // Number of records to be recorded
  float			hscale;		       // Horizontal scaling, ns/count
  float			_vscale[2];	     // Vertical scaling, volt/count , no longer used
  float			_voffset[2]; 	   // Vertical offset		, no longer used
  float			trig_11th;	     // Time at the 11-th trigger
  short			version;	       // version # to track updates
  short			channels;	       // number of active channels
  channel_setting	chan[4];	 // channel on or off
  short			scope_type;      //
  short			dummies[568];	   // Unused space
  char			wfmpre[512];	   // Waveform preamble from RTD720
} header_type;

#pragma pack (pop)

typedef struct plotFormat
{
  int   plotType;
  double padXMin;
  double padYMin;
  double padXMax;
  double padYMax;
  int   startBunch;
  int   endBunch;
  int   bunchesPerRow;
  int   momentOrder;
  bool plotSeparatrix;
  int   unitsType;                                 // This will be kdEdt, kdEphi, kdEphaseRF, etc.
  int   contourUnits;                              //Either time, phi, or dE
  std::vector< float>::size_type numberOfContours; //Should this be unsigned int?
  std::vector< float > contours;
};

enum plotTypes
{
  kImage,
  kData,
  kProj,
  kEmittanceHisto,
  kEmittance,
  kMachineStatsView,
  kReconStatsView,
  kMomentPhase,
  kMomentPhaseFFT,
  kMomentAmp,
  kMomentAmpFFT,
  kDiscrepancy,
  kDiscrepancyGraph
};

enum algorithmID
{
  kFCART_ID    = 0,
  kFCART2_ID   = 1,
  kDFCART_ID   = 2,
  kFCART3_ID   = 3,
  kTESTING_ID  = 4,
  kPROBART_ID  = 5,
  kPROBART2_ID = 6,
  kPROBART3_ID = 7
};

typedef struct Format_Config
{
  double hue;
  double start;
  double rotations;
  double gamma;

  int dipole_color;
  int quadrupole_color;

  bool InvertQ;

  int scheme_ID;

} Format_Config;

/// DJS STUFF for pre process May 2013
enum getBunchMidMethod { quick , allbunches, first, firstfull, fullbunches };
enum getProfileMethod  { simple, interp };


enum profileRequests
{
  kRequestDefault,
  kRequestPartialStart,
  kRequestFull1,
  kRequestFull2,
  kRequestFull3,
  kRequestFull4,
  kRequestFull5,
  kRequestFull6,
  kRequestPartialEnd,
  kRequestAll
};

typedef struct ppStruct
{
  float sigTrig;
  float dITrig;
  float bunchTrig;
  enum  getBunchMidMethod bunchMidMethod;
  enum  getProfileMethod  profileMethod;
  float bunchChargePartial;                /// from the first frame with data!
  float bunchChargeFull;                   /// from the first frame with data!
  int   ptsPerProfile;                     /// from the first frame with data!
  //ROOT::Math::Interpolation::Type bunchMidInterpType;
  //ROOT::Math::Interpolation::Type profInterpType;
} ppStruct;

typedef struct ppDataObjectInfo
{
  struct scopeParam sp;
  struct trigParam  ppTrigInfo; //NJE 7/8/13
  struct Machine_Config mc;
  struct ppStruct pps;
  float  bucketLength;
  int    sigStartFrame;
  int    sigEndFrame;
  int    numDIFrames;
  int    requestFrameStart,requestFrameEnd,requestBatchStart,requestBatchEnd;

  //std::vector<int> DISigFrames;
  std::vector<float > bunchMids;
  std::vector<int>    DISigFrames,dataStructures,requestBatchStartList, requestBatchEndList, requestFrameStartList, requestFrameEndList ;
  //std::vector<int> dataStructures; // holds encoded format of data
  std::vector< std::vector<int> > batchStartPos;
  std::vector< std::vector<int> > batchEndPos;
} ppDataObjectInfo;

const Format_Config     defaultFormat       = { 1.0, 0.5, -0.25, 0.8, 38, 46, true, 4 };
const Machine_Config    defaultMainInjector = { 588, 8e9, 938272064, 880e3, 52.809176e6, 21.838, 0.0 };
const Machine_Config    defaultRecycler     = { 588, 8e9, 938272064, 880e3, 52.8114e6, 21.838, 0.0 }; //Right now this is the same as MI
const Tomography_Config defaultTomography   = { 100, 2, 100, 0, 0.001, 1.0, kPROBART_ID, 100, 100 };


#endif


