#ifndef _DATAOBJECT_H
#define _DATAOBJECT_H

//stdlib includes
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <vector>

//NJE includes
//#include "TomographyStructDefaults.h"
#include "TomographyAlgorithm.h"
#include "TomographyStructs.h"
#include "TomographyConfig.h"
#include "TomographyMath.h"
#include "MapClass.h"

class dataObject
{
	private:

		complete_data_header_type objectHeader;
		std::vector< std::vector<float> > InputProfiles;
		std::vector< std::vector<float> > OutputProfiles;
		std::vector< std::vector<float> > OutputImages;
		std::vector< std::vector<double> > Discrepancies;

		void clearHeader();
		void hardPrep();

	public:

		dataObject( const Machine_Config machine, const Tomography_Config tomography );
		dataObject( const complete_data_header_type Config );
		~dataObject();
		void clear(); //Clears all the data vectors for a new fill. And header

		//The important stuff.
		void reconstructProfiles( const MapClass* maps, const int algo = kFCART_ID ); //Used for standalone mode. 
		void reconstructSingle( const MapClass* maps, const int bunch, const int algo = kFCART_ID ); //Used for program to allow updating of status bar.

		//Data Manipulation
		void padData(int halfPad);

		//Getters/Setters
		complete_data_header_type getConfig();
		profile_data_header_type  getDataAcquisition();
		Tomography_Config         getTomography();
		Machine_Config            getMachine();

		float *getData(    const int bunch );
		float *getImage(   const int bunch );
		float *getProfile( const int bunch );
		double *getDisc(    const int bunch );

		int getImageWidth(){ return objectHeader.tomography.pixelsPhi <= 0 ? objectHeader.dataAcquisition.ptsperprofile : objectHeader.tomography.pixelsPhi; };
		int getImageHeight(){ return  objectHeader.tomography.pixelsdE  <= 0 ? objectHeader.dataAcquisition.ptsperprofile : objectHeader.tomography.pixelsdE ; };

		std::vector<float>* getDataVec(    const int bunch );
		std::vector<float>* getImageVec(   const int bunch );
		std::vector<float>* getProfileVec( const int bunch );
		std::vector<double>* getDiscVec(   const int bunch );

		void setHeader(     const profile_data_header_type head );
		void setMachine(    const Machine_Config machine        );
		void setTomography( const Tomography_Config tomography  );

		bool setFramesToBest();

		void makeConsistent();
		void prepForTomography();
		
		// I/O
		bool readInPRPFile(  const char* filename );
		void readIn2DVector( const std::vector< std::vector< float > > input, const profile_data_header_type head );
		void bunchFromArray( const float* array, const int size, const int bunch );
		void bunchFromArray( const float* array, const profile_data_header_type* head );

		void writeImages(        const char* file );
		void writeProfiles(      const char* file );
		void writeDiscrepancies( const char* file );
		void writeFullData(      const char* file );
		void writeReconData(     const char* file );

		void printInputProfile( const int bunch ); //Really a debug function. prints to std out
		
		void printHeader();
		void printDataAcquisition();
		void printTomography();
		void printMachine();

		void killFrame(int frame); //erases a frame be careful with this one. 

};

#endif
